using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Input;

namespace MagGame
{

    public class ControllerHandler : Microsoft.Xna.Framework.GameComponent
    {
        private GamePadState[] prevGamePadsState = new GamePadState[4];
        private GamePadState[] gamePadsState;
        private Vector2 origin = new Vector2(0, 0);


        public ControllerHandler(Game game)
            : base(game)
        {
            //game.Services.AddService(typeof(IInputHandler), this);

            prevGamePadsState[0] = GamePad.GetState(PlayerIndex.One);
            prevGamePadsState[1] = GamePad.GetState(PlayerIndex.Two);
            gamePadsState = new GamePadState[2];

        }

        public override void Initialize()
        {
            base.Initialize();
        }

        public override void Update(GameTime gametime)
        {   // we need previous and current to detect when the button was just pressed
            // set our previous state to our new state
            prevGamePadsState[0] = gamePadsState[0];
            prevGamePadsState[1] = gamePadsState[1];

            // get our new state
            gamePadsState[0] = GamePad.GetState(PlayerIndex.One);
            gamePadsState[1] = GamePad.GetState(PlayerIndex.Two);

            if (gamePadsState[0].Buttons.Start == ButtonState.Pressed && prevGamePadsState[0].Buttons.Start == ButtonState.Released)
            {
                Program.getGame().pause();
            }


            if (gamePadsState[0].Buttons.Back == ButtonState.Pressed && prevGamePadsState[0].Buttons.Back == ButtonState.Released)
            {
                //Program.getGame().initGameObjects();
                Program.getGame().Reset();
            }

            if (gamePadsState[0].Buttons.A == ButtonState.Pressed && prevGamePadsState[0].Buttons.A == ButtonState.Released)
            {
                Program.getGame().progressScreen();
            }

            if (Program.getGame().isPaused())   // only need to check for unpause or reset
                return;

            // ** first check player one input
            // if player 1 is pressing A
            if (gamePadsState[0].Buttons.A == ButtonState.Pressed)// && prevGamePadsState[0].Buttons.A == ButtonState.Released)
            {
                //p1hero.SetSpeed(new Vector3(1f, 0, 0));
                //  p1root.setXformInfo(xf);
            }


            // Clear all the points if magnet is not active
            if (prevGamePadsState[0].Triggers.Right <= 0.1f && prevGamePadsState[0].Triggers.Left <= 0.1f)
                Program.getGame().getPlayerOne().ClearPoints(); 

            // consumption rate needs to be changed even if its 0
            if (gamePadsState[0].Triggers.Right == 0 && gamePadsState[0].Triggers.Left == 0)
            {
                Program.getGame().getPlayerOne().changeConsumption(gamePadsState[0].Triggers.Right);
                Program.getGame().P1LetGoOfObject();
            }
            // if attraction magnet is pushed conusme fuel and give hero the magnet power
            else if (gamePadsState[0].Triggers.Right >= 0.1f)
            {
                // only the trigger pushed down the most used, and the hero must have some fuel
                if (Program.getGame().getPlayerOne().hasFuel())
                {
                    Program.getGame().getPlayerOne().SuctionAnimation(gamePadsState[0].Triggers.Right);
                    Program.getGame().getPlayerOne().changeConsumption(gamePadsState[0].Triggers.Right);
                    // TODO: give the hero detraction magnet power
                    Program.getGame().P1PullMagnetHandler(gamePadsState[0].Triggers.Right);
                    
                }
            }
            // if push magnet is enabled, consume fuel and apply power
            else if (gamePadsState[0].Triggers.Left >= 0.1f)
            {
                Program.getGame().getPlayerOne().BlowAnimation(gamePadsState[0].Triggers.Left);
                Program.getGame().getPlayerOne().changeConsumption(gamePadsState[0].Triggers.Left);
                Program.getGame().P1PushMagnetHandler(gamePadsState[0].Triggers.Left);
                
            }


            // Direction controls
            if (!gamePadsState[0].ThumbSticks.Left.Equals(origin))
            {
                // velocity is the y axis of left controller
                Program.getGame().getPlayerOne().SetSpeed(gamePadsState[0].ThumbSticks.Left.Y * -3f);
                Program.getGame().getPlayerOne().setRotation(gamePadsState[0].ThumbSticks.Left.X * -3f);
                Program.getGame().getPlayerOne().PlayAnimation(0.025f);

            }
            else
            {
                Program.getGame().getPlayerOne().SetSpeed(0);
                //Program.getGame().getPlayerOne().setRotation(0);
            }


            if (!gamePadsState[0].ThumbSticks.Right.Equals(origin))
            {
                Program.getGame().getPlayerOne().setMagnetRotation(gamePadsState[0].ThumbSticks.Right.X * -3f);
            }

            // ZOOM control
            if (gamePadsState[0].Buttons.Y == ButtonState.Pressed &&
                prevGamePadsState[0].Buttons.Y == ButtonState.Released)
                Program.getGame().ChangeZoomLevel(0);


            // DEBUGG
            if (gamePadsState[0].DPad.Left == ButtonState.Pressed)
                Program.getGame().setHealthLvl(-1);
            if (gamePadsState[0].DPad.Right == ButtonState.Pressed)
                Program.getGame().setHealthLvl(1);

            GamePad.SetVibration(PlayerIndex.One, gamePadsState[0].Triggers.Left, gamePadsState[0].Triggers.Right);
            /*GamePad.SetVibration(PlayerIndex.One, prevGamePadsState[0].Triggers.Left,
                prevGamePadsState[0].Triggers.Right);*/


            /////////////////////////////////
            // ** check for player two input
            // if player 2 is pressing A
            //
            // 
            // 

            // consumption rate needs to be changed even if its 0
            if (gamePadsState[1].Triggers.Right == 0 && gamePadsState[1].Triggers.Left == 0)
            {
                Program.getGame().getPlayerTwo().changeConsumption(gamePadsState[1].Triggers.Right);
                Program.getGame().P2LetGoOfObject();
            }

        
                
            // Clear all the points if magnet is not active
            if (prevGamePadsState[1].Triggers.Right <= 0.1f && prevGamePadsState[1].Triggers.Left <= 0.1f)
                Program.getGame().getPlayerTwo().ClearPoints();

            // if attraction magnet is pushed conusme fuel and give hero the magnet power
            else if (gamePadsState[1].Triggers.Right >= 0.1f)
            {
                // only the trigger pushed down the most used, and the hero must have some fuel
                if (Program.getGame().getPlayerTwo().hasFuel())
                {
                    Program.getGame().getPlayerTwo().changeConsumption(gamePadsState[1].Triggers.Right);
                    // TODO: give the hero detraction magnet power
                    Program.getGame().P2PullMagnetHandler(gamePadsState[1].Triggers.Right);
                    Program.getGame().getPlayerTwo().SuctionAnimation(gamePadsState[1].Triggers.Right);
                }
            }
            // if push magnet is enabled, consume fuel and apply power
            else if (gamePadsState[1].Triggers.Left >= 0.1f)
            {
                Program.getGame().getPlayerTwo().changeConsumption(gamePadsState[1].Triggers.Left);
                Program.getGame().P2PushMagnetHandler(gamePadsState[1].Triggers.Left);
                Program.getGame().getPlayerTwo().BlowAnimation(gamePadsState[1].Triggers.Left);
            }


            // Direction controls
            if (!gamePadsState[1].ThumbSticks.Left.Equals(origin))
            {
                // velocity is the y axis of left controller
                Program.getGame().getPlayerTwo().SetSpeed(gamePadsState[1].ThumbSticks.Left.Y * -3f);
                Program.getGame().getPlayerTwo().setRotation(gamePadsState[1].ThumbSticks.Left.X * -3f);
                Program.getGame().getPlayerTwo().PlayAnimation(0.025f);

            }
            else
            {
                Program.getGame().getPlayerTwo().SetSpeed(0);
                //Program.getGame().getPlayerOne().setRotation(0);
            }


            if (!gamePadsState[1].ThumbSticks.Right.Equals(origin))
            {
                Program.getGame().getPlayerTwo().setMagnetRotation(gamePadsState[1].ThumbSticks.Right.X * -3f);
            }


            // ZOOM control
            if (gamePadsState[1].Buttons.Y == ButtonState.Pressed &&
                prevGamePadsState[1].Buttons.Y == ButtonState.Released)
                Program.getGame().ChangeZoomLevel(1);

            GamePad.SetVibration(PlayerIndex.Two, gamePadsState[1].Triggers.Left,
                    gamePadsState[1].Triggers.Right);

        }

    }
}