using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
using System.Runtime.InteropServices;

namespace MagGame
{
    public class MagGame : Microsoft.Xna.Framework.Game
    {
        // new xna stuff
        public static GraphicsDeviceManager MyGraphics;

        public static GraphicsDeviceManager MyGraphicsManager
        {
            get { return MyGraphics; }
            set { MyGraphics = value; }
        }
        public static BasicEffect MyBasicEffect;
        public static BasicEffect MyEffect
        {
            get { return MyBasicEffect; }
            set { MyBasicEffect = value; }
        }
        public static VertexDeclaration MyVertexDeclaration; // make global so drawhelper can grab it
        public static VertexDeclaration MyVertexDec
        {
            get { return MyVertexDeclaration; }
            set { MyVertexDeclaration = value; }
        }

        #region View and effects
        private EffectPool myEffectPool;
        private ContentManager content;
        private Viewport leftViewport;
        private Viewport rightViewport;
        private Viewport bottomViewport;
        private Viewport btmframeViewport;
        private Viewport separatorViewport;
        private Viewport horizontalbar;
        private Viewport pauseMenuViewport;
        private Viewport mainMenuViewport;
        #endregion

        #region Textures
        public static Texture2D texture;
        public static Texture2D tex_tire;
        public static Texture2D tex_hardbody;
        public static Texture2D tex_carbon;
        public static Texture2D tex_shiny;
        public static Texture2D tex_shinymetal;
        public static Texture2D tex_chinared;
        public static Texture2D tex_healthbar;
        public static Texture2D tex_pausemenu;
        public static Texture2D tex_background;
        public static Texture2D tex_hole;
        public static Texture2D tex_splash;
        public static Texture2D tex_instruction;
        public static Texture2D tex_redbar;
        public static Texture2D tex_bluebar;
        public static Texture2D tex_normshot;
        public static Texture2D tex_background2;
        public static Texture2D tex_background3;
        public static Texture2D tex_redspiral;
        public static Texture2D tex_p2wins;
        public static Texture2D tex_p1wins;
        public static Texture2D tex_bodyred;
        public static Texture2D tex_bodyblue;
        public static Texture2D tex_yellowtire;
        private PauseMenuImage pauseMenu;
        private InstructionsMenuImage instructionMenu;
        private SplashMenuImage splashMenu;
        private P1GameOverMenuImage p1gameovermenu;
        private P2GameOverMenuImage p2gameovermenu;
        #endregion

        private ControllerHandler inputHandler;

        private UWB_Clock mStopWatch;

        private float mWorldWidth;
        private float mWorldHeight;

        private UWB_XNADrawHelper mDrawHelper;

        private UWB_BoundingBox m_worldbounds;
        private UWB_BoundingBox m_leftview;
        private UWB_BoundingBox m_rightview;


        private UWB_PrimitiveRectangle background;

        private Hero playerOne;
        private Hero playerTwo;

        //float elapsedseconds;
        //bool animate;

        bool m_pause;
        bool isSplashPage;
        bool isMainPage;
        bool isP1Gameover;
        bool isP2Gameover;
        float m_P1zoomlvl;
        float m_P2zoomlvl;

        HorizontalBar m_Progressbar;

        ObstacleManager m_Obstacles;
        int m_stage;
        int m_P1life;
        int m_P2life;
        Vector3 m_P1GravityCenter;
        Vector3 m_P2GravityCenter;
        SoundManager soundManager;  // NEW ENTRY

        public MagGame()
        {
            inputHandler = new ControllerHandler(this);

            MyGraphics = new GraphicsDeviceManager(this);
            content = new ContentManager(Services);
            pauseMenu = new PauseMenuImage();
            instructionMenu = new InstructionsMenuImage();
            splashMenu = new SplashMenuImage();
            p1gameovermenu = new P1GameOverMenuImage();
            p2gameovermenu = new P2GameOverMenuImage();
            // NEW ENTRY sound management stuff.
            soundManager = new SoundManager(this, "sounds");
            //string[] playList = { "Tribeta", "Tribeta", "Tribeta" };
            //soundManager.StartPlayList(playList);
            Components.Add(soundManager);
            // end new stuff
            m_pause = true;
            isSplashPage = true;
            isMainPage = false;
            isP1Gameover = false;
            isP2Gameover = false;

            initGameObjects();
            #region Worldcoord and View initializations
            mWorldWidth = 450;
            mWorldHeight = 132;

           
            Vector3 min = new Vector3(-50.0f, -50.0f, 0.0f);
            Vector3 max = new Vector3(500.0f, 182f, 0.0f);
            background = new UWB_PrimitiveRectangle();
            background.SetCorners(min, max);
            background.EnableBlending(true);
            background.EnableTexturing(true);
            background.SetTextureFilename("background");
            background.setShadeMode(eShadeMode.smFlat);
            background.setFlatColor(Color.White);
            
            mDrawHelper = new UWB_XNADrawHelper();
            mDrawHelper.InitializeModelTransform();

            mStopWatch = new UWB_Clock();


            // Set our world coordinates
            Vector3 corner1 = new Vector3(0, 0, 0);
            Vector3 corner2 = new Vector3(mWorldWidth, mWorldHeight, 0);
            m_worldbounds = new UWB_BoundingBox(corner1, corner2);

            //corner1 = new Vector3(12.2f, 7, 0);
            //corner2 = new Vector3(41.2f, 44, 0);
            UWB_BoundingBox p1bound = playerOne.getRootNode().getBounds(mDrawHelper, false);
            Vector3 p1ctr = p1bound.GetCenter();

            corner1 = new Vector3(0, 15, 0);
            corner2 = new Vector3(39, 71, 0);
            //corner1 = new Vector3(p1ctr.X - 19.5f, p1ctr.Y - 25, 0);
            //corner2 = new Vector3(p1ctr.X + 19.5f, p1ctr.Y + 25, 0);
            m_leftview = new UWB_BoundingBox(corner1, corner2);

            corner1 = new Vector3(300, 15, 0);
            corner2 = new Vector3(261, 71, 0);

            m_rightview = new UWB_BoundingBox(corner1, corner2);
            #endregion

            m_Progressbar = new HorizontalBar();


            m_P1zoomlvl = 1.0f;
            m_P2zoomlvl = 1.0f;

            m_Obstacles = new ObstacleManager();

            // Player and game statistics
            m_stage = 1;
            m_P1life = 3;
            m_P2life = 3;
            m_P1GravityCenter = new Vector3();
            m_P2GravityCenter = new Vector3();

        }

        public void initGameObjects()
        {
            if (!m_pause)   // if were not paused then you cant restart.
                return;

            resourceList = new List<ResourceObject>();
            resourceGenerator = new ResrouceManager();

            P1currentObj = null;
            P2currentObj = null;
            playerOne = new Hero(1);
            playerTwo = new Hero(2);
            UWB_XformInfo xf = playerOne.getRootNode().getXformInfo();
            xf.SetTranslation(new Vector3(-67.2f, 43, 0));
            xf.SetRotationInDegrees(-90);
            playerOne.getRootNode().setXformInfo(xf);

            xf = playerTwo.getRootNode().getXformInfo();
            xf.SetTranslation(new Vector3(338.3f, 43, 0));
            xf.SetRotationInDegrees(90);
            playerTwo.getRootNode().setXformInfo(xf);

            //ResetP1Stat();
            //ResetP2Stat();


        }

        protected override void Initialize()
        {
            base.Initialize();
            this.Components.Add(inputHandler);

            m_Progressbar.Initialize();
            m_Obstacles.Initialize(m_stage);

        }


        /// <summary>
        /// Load your graphics content.  If loadAllContent is true, you should
        /// load content from both ResourceManagementMode pools.  Otherwise, just
        /// load ResourceManagementMode.Manual content.
        /// </summary>
        /// <param name="loadAllContent">Which type of content to load.</param>
        protected override void LoadGraphicsContent(bool loadAllContent)
        {

            if (loadAllContent)
            {
                //Initialize Effect
                myEffectPool = new EffectPool();
                MyBasicEffect = new BasicEffect(MyGraphics.GraphicsDevice, myEffectPool);
                MyVertexDeclaration = new VertexDeclaration(MyGraphics.GraphicsDevice,
                   VertexPositionColorTexture.VertexElements);

            }
            #region Texture stuff
            tex_tire = content.Load<Texture2D>("Content/tire");
            tex_hardbody = content.Load<Texture2D>("Content/hardbody");
            tex_carbon = content.Load<Texture2D>("Content/carbon");
            tex_shiny = content.Load<Texture2D>("Content/shiny");
            tex_shinymetal = content.Load<Texture2D>("Content/shinymetal");
            tex_chinared = content.Load<Texture2D>("Content/ChinaRed");
            tex_healthbar = content.Load<Texture2D>("Content/b6") as Texture2D;
            tex_pausemenu = content.Load<Texture2D>("Content/pausemenu") as Texture2D;
            tex_background = content.Load<Texture2D>("Content/background") as Texture2D;
            tex_hole = content.Load<Texture2D>("Content/transparentSpiral") as Texture2D;
            tex_instruction = content.Load<Texture2D>("Content/instructions") as Texture2D;
            tex_splash = content.Load<Texture2D>("Content/splash") as Texture2D;
            tex_bluebar = content.Load<Texture2D>("Content/health_blue") as Texture2D;
            tex_redbar = content.Load<Texture2D>("Content/health_red") as Texture2D;
            tex_normshot = content.Load<Texture2D>("Content/BasicShot") as Texture2D;
            tex_background2 = content.Load<Texture2D>("Content/background2") as Texture2D;
            tex_background3 = content.Load<Texture2D>("Content/background4") as Texture2D;
            tex_redspiral = content.Load<Texture2D>("Content/redSpiral") as Texture2D;
            tex_p1wins = content.Load<Texture2D>("Content/P1Wins") as Texture2D;
            tex_p2wins = content.Load<Texture2D>("Content/P2Wins") as Texture2D;
            tex_bodyred = content.Load<Texture2D>("Content/bodyred") as Texture2D;
            tex_bodyblue = content.Load<Texture2D>("Content/bodyblue") as Texture2D;
            tex_yellowtire = content.Load<Texture2D>("Content/yellow_wheel") as Texture2D;
            #endregion

            #region View Stuff
            MyBasicEffect.World = Matrix.Identity;
            MyBasicEffect.View = Matrix.Identity;
            MyBasicEffect.Projection = Matrix.Identity;

            leftViewport = MyGraphics.GraphicsDevice.Viewport;
            rightViewport = leftViewport;
            bottomViewport = leftViewport;
            separatorViewport = leftViewport;
            mainMenuViewport = leftViewport;
            pauseMenuViewport = leftViewport;
            pauseMenuViewport.Width = (int)(pauseMenuViewport.Width * .75f);
            pauseMenuViewport.Height = (int)(pauseMenuViewport.Height * .75f);
            pauseMenuViewport.X = 100;
            pauseMenuViewport.Y = 100;

            leftViewport.Width = leftViewport.Width / 2;

            separatorViewport.X = leftViewport.Width - 1;
            separatorViewport.Width = 3;

            rightViewport.X = leftViewport.Width + 1;
            rightViewport.Width = rightViewport.Width / 2;

            /*bottomViewport.Height = bottomViewport.Height / 5;
            bottomViewport.Width = bottomViewport.Width / 2;
            bottomViewport.Y = (int)getWorldBound().GetMax().Y; */
            bottomViewport.Height = MyGraphics.GraphicsDevice.Viewport.Height / 5;
            bottomViewport.Width = MyGraphics.GraphicsDevice.Viewport.Width / 2;
            bottomViewport.X = leftViewport.Width / 2;
            bottomViewport.Y = MyGraphics.GraphicsDevice.Viewport.Height - (int)(getWorldBound().Height());


            btmframeViewport.Height = MyGraphics.GraphicsDevice.Viewport.Height / 5 + 10;
            btmframeViewport.Width = MyGraphics.GraphicsDevice.Viewport.Width / 2 + 10;
            btmframeViewport.X = leftViewport.Width / 2 - 5;
            btmframeViewport.Y = MyGraphics.GraphicsDevice.Viewport.Height - (int)(getWorldBound().Height()) - 5;

            horizontalbar.Height = MyGraphics.GraphicsDevice.Viewport.Height / 20;
            horizontalbar.Width = MyGraphics.GraphicsDevice.Viewport.Width - 60;
            horizontalbar.X = 30;
            horizontalbar.Y = 20;
            #endregion


        }


        /// <summary>
        /// Unload your graphics content.  If unloadAllContent is true, you should
        /// unload content from both ResourceManagementMode pools.  Otherwise, just
        /// unload ResourceManagementMode.Manual content.  Manual content will get
        /// Disposed by the GraphicsDevice during a Reset.
        /// </summary>
        /// <param name="unloadAllContent">Which type of content to unload.</param>
        protected override void UnloadGraphicsContent(bool unloadAllContent)
        {
            if (unloadAllContent == true)
            {
                content.Unload();
            }
        }

        public UWB_BoundingBox getWorldBound()
        {
            return m_worldbounds;
        }

        // move to next screen, if were on splash / main menu
        public void progressScreen()
        {
            if (isMainPage)
            {
                isMainPage = false;
                m_pause = false;
            }
            if (isSplashPage)
            {
                isSplashPage = false;
                isMainPage = true;
            }
            if (isP1Gameover)
            {
                isP1Gameover = false;
            }
            if (isP2Gameover)
            {
                isP2Gameover = false;
            }
        }


        public void MoveHeroTo(float X, float Y)
        {
            UWB_XformInfo xform = new UWB_XformInfo();
            Vector3 center = new Vector3();

            center = playerOne.getRootNode().getXformInfo().GetPivot();
            xform = playerOne.getRootNode().getXformInfo();
            xform.SetTranslation(new Vector3(X - center.X, Y - center.Y, 0));
            playerOne.getRootNode().setXformInfo(xform);
        }

        public UWB_SceneNode getRootNode()
        {
            return playerOne.getRootNode();
        }

        public bool isOutsideWorldBounds(Vector3 position)
        {
            if (position.X < 0 || position.X > mWorldWidth)
                return true;
            if (position.Y < 0 || position.Y > mWorldHeight)
                return true;

            return false;
        }

        protected override void Update(GameTime gameTime)
        {
            inputHandler.Update(gameTime);
                    if (!soundManager.IsPlaying("Tribeta"))
                        soundManager.Play("Tribeta"); // NEW ENTRY
            if (m_pause)
                return;

            playerOne.Update((float)gameTime.ElapsedGameTime.TotalSeconds);
            playerTwo.Update((float)gameTime.ElapsedGameTime.TotalSeconds);

            this.ComputeBoundPosition(playerOne, ref m_leftview, m_P1zoomlvl);
            this.ComputeBoundPosition(playerTwo, ref m_rightview, m_P2zoomlvl);

            UpdateSimulation(gameTime);

        }

        public void UpdateSimulation(GameTime gameTime)
        {
            updateResources((float)gameTime.ElapsedGameTime.TotalSeconds);
            //playerOne.HeroWorldBoundResponse(0);
            BoundaryCheck(ref playerOne);
            BoundaryCheck(ref playerTwo);
            HeroToHeroCollisionHandler();
            HoleCollisionCheck();
            
            UpdatePlayerCondition();
            CheckWinningCondition();

            for (int i = 0; i < resourceList.Count; i++)
            {
                resourceList[i].update(0.15f);
                if (resourceList[i].getPosition().X < 0 || resourceList[i].getPosition().X > mWorldWidth)
                {
                    resourceGenerator.deleteResource(); // decrement counter
                    resourceList.RemoveAt(i);
                    // NEW ENTRY
                    break;  // NEW ENTRY avoids crash is last object hill wall
                }
                if (resourceList[i].getPosition().Y < 0 || resourceList[i].getPosition().Y > mWorldHeight)
                {
                    resourceGenerator.deleteResource(); // decrement counter
                    resourceList.RemoveAt(i);
                }
            }

            m_Obstacles.AnimateAll(0.05f);
            // DEBUG
            playerOne.isInMagnetWaveBounds(new Vector3(0, 0, 0), new Vector3(1, 1, 1), mDrawHelper);

        }

        protected override void Draw(GameTime gameTime)
        {
            mDrawHelper.InitializeModelTransform();
            MyGraphics.GraphicsDevice.VertexDeclaration = MyVertexDeclaration;
            MyGraphics.GraphicsDevice.RenderState.CullMode = CullMode.None;
            MyGraphics.GraphicsDevice.RenderState.FillMode = FillMode.Solid;
            MyGraphics.GraphicsDevice.RenderState.PointSpriteEnable = true;
            //  MyBasicEffect.VertexColorEnabled = true;
            //  MyBasicEffect.TextureEnabled = true;
            //  MyBasicEffect.Texture = texture;
            MyBasicEffect.CommitChanges();
            MyBasicEffect.Begin();
            MyBasicEffect.CurrentTechnique.Passes[0].Begin();


            for (int i = 0; i < 6; i++) // draw to the 3 viewports
                DrawScene(i);   // tell DrawScene which viewport to draw to
            if (m_pause)
                DrawScene(6);   // draw the pause screen overlay.

            base.Draw(gameTime);

            MyBasicEffect.CurrentTechnique.Passes[0].End();
            MyBasicEffect.End();
        }


        private void DrawScene(int playerNumber)
        {
            // This part handles drawing of full screen menu items
            eLevelofDetail lod = mDrawHelper.getLod();
            if (isSplashPage)
            {
                UWB_BoundingBox bb = new UWB_BoundingBox(new Vector3(.5f, .5f, 0), 1,1,0);
                SetUpView(ref bb);
                MyGraphics.GraphicsDevice.Viewport = mainMenuViewport;
                MyGraphics.GraphicsDevice.Clear(Color.Aquamarine);
                splashMenu.draw(mDrawHelper);
                return;
            }
            if (isMainPage)
            {
                MyGraphics.GraphicsDevice.Viewport = mainMenuViewport;
                MyGraphics.GraphicsDevice.Clear(Color.Aquamarine);
                instructionMenu.draw(mDrawHelper);
                return;
            }
            if (isP1Gameover)
            {
                UWB_BoundingBox bb = new UWB_BoundingBox(new Vector3(.5f, .5f, 0), 1, 1, 0);
                SetUpView(ref bb);
                MyGraphics.GraphicsDevice.Viewport = mainMenuViewport;
                MyGraphics.GraphicsDevice.Clear(Color.Aquamarine);
                p1gameovermenu.draw(mDrawHelper);
                return;
            }
            if (isP2Gameover)
            {
                UWB_BoundingBox bb = new UWB_BoundingBox(new Vector3(.5f, .5f, 0), 1, 1, 0);
                SetUpView(ref bb);
                MyGraphics.GraphicsDevice.Viewport = mainMenuViewport;
                MyGraphics.GraphicsDevice.Clear(Color.Aquamarine);
                p2gameovermenu.draw(mDrawHelper);
                return;
            }

            // Drawing of our game views
            switch (playerNumber)
            {
                case 0:
                    {
                        MyGraphics.GraphicsDevice.Viewport = leftViewport;
                        SetUpView(ref m_leftview);
                        MyGraphics.GraphicsDevice.Clear(Color.Wheat);
                    }
                    break;
                case 1:
                    {
                        MyGraphics.GraphicsDevice.Viewport = rightViewport;
                        SetUpView(ref m_rightview);
                        MyGraphics.GraphicsDevice.Clear(Color.Wheat);
                    }
                    break;
                case 2:  // divider
                    {
                        MyGraphics.GraphicsDevice.Viewport = separatorViewport;
                        MyGraphics.GraphicsDevice.Clear(Color.Black);
                        return;
                    }
                case 3:
                    {
                        MyGraphics.GraphicsDevice.Viewport = btmframeViewport;
                        MyGraphics.GraphicsDevice.Clear(Color.Black);
                        return;
                    }
                case 4: // the main zoomed out viewport
                    {
                        MyGraphics.GraphicsDevice.Viewport = bottomViewport;
                        SetUpView(ref m_worldbounds);
                        MyGraphics.GraphicsDevice.Clear(Color.Wheat);
                        // DEBUG
                        m_leftview.Draw(mDrawHelper);
                        m_rightview.Draw(mDrawHelper);

                    }
                    break;
                case 5:
                    {
                        UWB_BoundingBox box = m_Progressbar.GetBarBorder();
                        MyGraphics.GraphicsDevice.Viewport = horizontalbar;
                        MyGraphics.GraphicsDevice.Clear(Color.Blue);
                        SetUpView(ref box);
                        m_Progressbar.Draw(lod, mDrawHelper);
                        return;
                    }
                case 6:
                    {
                        MyGraphics.GraphicsDevice.Viewport = pauseMenuViewport;
                        MyGraphics.GraphicsDevice.Clear(Color.Aquamarine);
                        pauseMenu.draw(mDrawHelper);
                        return;
                    }
                default:
                    break;
            }

            background.draw(lod, mDrawHelper);
            m_worldbounds.Draw(mDrawHelper);
            m_Obstacles.Draw(lod, mDrawHelper);
            playerOne.draw(lod, mDrawHelper);
            playerTwo.draw(lod, mDrawHelper);
            drawResources();    // draw all world resources
            //m_Obstacles.Draw(lod, mDrawHelper);

            //ShowMagnetWave(ref playerOne);
            // DEBUGGING
            //     playerOne.DrawHeroBound(mDrawHelper);
            //      playerTwo.DrawHeroBound(mDrawHelper);

            /*foreach (UWB_Primitive aball in mBalls)
                aball.draw(lod, mDrawHelper);
            foreach (UWB_Primitive aexp in mExplosion)
                aexp.draw(lod, mDrawHelper);*/
        }

        // TEMPORARY FIX FOR DEMO
        public void SetUpView(ref UWB_BoundingBox view)
        {
            // some noob forgot to setup draw area.. no wonder it wouldn't draw
            UWB_BoundingBox m_WCWindow = view;

            Matrix world2ndc;
            Matrix toSize;
            Matrix toPlace;

            Vector3 center = m_WCWindow.GetCenter();

            toPlace = Matrix.CreateTranslation(-center);
            toSize = Matrix.CreateScale(2.0f / m_WCWindow.Width(), 2.0f / m_WCWindow.Height(), 1.0f);
            world2ndc = toPlace * toSize;

            MyBasicEffect.View = world2ndc;
            MyBasicEffect.CommitChanges();
        }

        public void ComputeBoundPosition(Hero player, ref UWB_BoundingBox camera, float m_zoomlvl)
        {
            UWB_XformInfo xf = player.getRootNode().getXformInfo();
            Vector3 trans = xf.GetTranslation();
            Vector3 pivot = xf.GetPivot();
            Vector3 t = trans + pivot;

            Vector3 corner1 = new Vector3(t.X - 25f * m_zoomlvl, t.Y - 36.5f * m_zoomlvl, 0.0f);
            Vector3 corner2 = new Vector3(t.X + 25f * m_zoomlvl, t.Y + 40.5f * m_zoomlvl, 0.0f);

            camera.SetCorners(corner1, corner2);
        }

        public Hero getPlayerOne()
        {
            return playerOne;
        }

        public Hero getPlayerTwo()
        {
            return playerTwo;
        }

        #region Hero Collision stuff
        public void BoundaryCheck(ref Hero player)
        {
            UWB_BoundingCircle heroBound = player.GetHeroBound(mDrawHelper);
            Vector3 center = heroBound.Center;
            float radius = heroBound.Radius;

            if (center.X - m_worldbounds.GetMin().X < 6.0f)
            {
                player.HeroWorldBoundResponse(0);
            }
            else if (center.X + radius > mWorldWidth)
            {
                player.HeroWorldBoundResponse(1);
            }
            else if (center.Y - m_worldbounds.GetMin().Y < 6.0f)
            {
                player.HeroWorldBoundResponse(2);
            }
            else if (center.Y + radius > mWorldHeight)
            {
                player.HeroWorldBoundResponse(3);
            }
        }

        public void HeroToHeroCollisionHandler()
        {
            UWB_BoundingVolume other = (UWB_BoundingVolume)playerTwo.GetHeroBound(mDrawHelper);
            // Check P1 body against P2 body and P2 manget
            if (playerOne.GetHeroBound(mDrawHelper).Intersects(ref other))
            {
                Vector3 newP1Velocity = new Vector3();
                newP1Velocity.X += .5f * (playerOne.GetHeroBound(mDrawHelper).Center.X -
                    ((UWB_BoundingCircle)other).Center.X);
                newP1Velocity.Y += .5f * (playerOne.GetHeroBound(mDrawHelper).Center.Y -
                    ((UWB_BoundingCircle)other).Center.Y);

                Vector3 newP2Velocity = new Vector3();
                newP2Velocity.X += .5f * (((UWB_BoundingCircle)other).Center.X -
                (playerOne.GetHeroBound(mDrawHelper).Center.X));
                newP2Velocity.Y += .5f * (((UWB_BoundingCircle)other).Center.Y -
                (playerOne.GetHeroBound(mDrawHelper).Center.Y));

                playerOne.setVelocityDeltaModel(newP1Velocity);
                playerTwo.setVelocityDeltaModel(newP2Velocity);
                // playerOne.getRootNode().getXformInfo().SetTranslation(new Vector3(-80f, 23, 0));
            }
        }

        // handles projectile to hero collisions.
        // to detect if a projectile hit P2, it will see check if P1's current object is touching
        // P2. if it is, P2 just got it. called from updateResources()
        public void ProjectileToP2Collision()
        {
            if (P1currentObj != null)
            {
                UWB_BoundingCircle p2Bound = (UWB_BoundingCircle)playerTwo.GetHeroBound(mDrawHelper);
                //  P1currentObj.updateHomingPosition(p2Bound.GetCenter());
                UWB_BoundingVolume objBound = (UWB_BoundingVolume)P1currentObj.getObjectBound();
                if (p2Bound.Intersects(ref objBound))
                {
                    Vector3 collideLoc = P1currentObj.getPosition();
                    resourceGenerator.deleteResource(); // decrement counter
                    resourceList.Remove(P1currentObj);
                    if (P1currentObj.typeOfResource == eResourceType.straightShot)
                        m_Progressbar.CurrentValue += 10;
                    else
                        m_Progressbar.CurrentValue += 15;
                    // NEW ENTRY
                    if (P1currentObj.typeOfResource == eResourceType.heatSeeker)
                        soundManager.Play("BigBulletCollision");
                    if (P1currentObj.typeOfResource == eResourceType.straightShot)
                        soundManager.Play("littleBulletCollision");
                    P1currentObj = null;
                    // note P1currentObj doesnt get destroyed, i added this in new version?
                    // for hp bar fix.
                    // END ENTRY

                }
                if (P1currentObj != null && P1currentObj.isFired && P1currentObj.typeOfResource == eResourceType.heatSeeker)
                {
                    P1currentObj.updateHomingPosition(p2Bound.GetCenter());
                }
            }
        }

        public void ProjectileToP1Collision()
        {
            if (P2currentObj != null)
            {
                UWB_BoundingCircle p1Bound = (UWB_BoundingCircle)playerOne.GetHeroBound(mDrawHelper);
                //  P1currentObj.updateHomingPosition(p2Bound.GetCenter());
                UWB_BoundingVolume objBound = (UWB_BoundingVolume)P2currentObj.getObjectBound();
                if (p1Bound.Intersects(ref objBound))
                {
                    Vector3 collideLoc = P2currentObj.getPosition();
                    resourceGenerator.deleteResource(); // decrement counter
                    resourceList.Remove(P2currentObj);
                    if (P2currentObj.typeOfResource == eResourceType.straightShot)
                        m_Progressbar.CurrentValue -= 10;
                    else
                        m_Progressbar.CurrentValue -= 15;
                    // NEW ENTRY
                    if (P2currentObj.typeOfResource == eResourceType.heatSeeker)
                        soundManager.Play("BigBulletCollision");
                    if (P2currentObj.typeOfResource == eResourceType.straightShot)
                        soundManager.Play("littleBulletCollision");
                    P2currentObj = null;
                    // note P1currentObj doesnt get destroyed, i added this in new version?
                    // for hp bar fix.
                    // END ENTRY
                }
                ////if (P1currentObj.typeOfResource == eResourceType.straightShot)

                if (P2currentObj != null && P2currentObj.isFired && P2currentObj.typeOfResource == eResourceType.heatSeeker)
                {
                    P2currentObj.updateHomingPosition(p1Bound.GetCenter());
                }
            }
        }
        #endregion

        #region Resource management
        // new resources are created as stationary objects in the world
        // hero can interact with the objects freely.


        // MEMBERS: instantiate these in constructor.

        // list of all resources that exist in the world.
        private List<ResourceObject> resourceList;
        private ResrouceManager resourceGenerator;
        private ResourceObject P1currentObj;
        private ResourceObject P2currentObj;

        // call from Update()
        public void updateResources(float elapsedSeconds)
        {
            if (resourceGenerator.isCreateResource(elapsedSeconds))
            {
                ResourceObject newResource = new ResourceObject(m_worldbounds.GetMax());
                resourceList.Add(newResource);
            }

            ProjectileToP1Collision();
            ProjectileToP2Collision();
        }

        // call from draw()
        public void drawResources()
        {
            foreach (ResourceObject res in resourceList)
                res.resourceDraw(mDrawHelper);
        }


        #endregion

        #region Player MagnetHandling
        public void P1ActivatePullMagnet(float speed)
        {
            Vector3 center;
            Vector3 point;
            // MAGNET TO RESOURCE HANDLING
            for (int i = 0; i < resourceList.Count; i++)
            {
                center = resourceList[i].getObjectBound().Center;
                point = new Vector3(center.X + resourceList[i].getObjectBound().Radius,
                    center.Y + resourceList[i].getObjectBound().Radius, 0);
                if (playerOne.isInHeroCoverageArea(center))
                {
                    if (playerOne.isInMagnetWaveBounds(center, point, mDrawHelper))
                    {
                        P1currentObj = resourceList[i];
                        break;
                    }
                }
            } // END OF MAGNET TO RESOURCE HANDLING
            // P1 MAGNET TO P2 BODY HANDLING
            // this will first check if the magnet bound is touching the other hero.
            // if it is, we will take the center point of p1 and p2, and change p2's
            // velocity so that he moves towards p1.
            UWB_BoundingCircle p2Bound = (UWB_BoundingCircle)playerTwo.GetHeroBound(mDrawHelper);
            Vector3 p2radiusPoint = new Vector3(p2Bound.Center.X, p2Bound.GetCenter().Y +
                p2Bound.Radius, 0);
            // if player one's magnet is touching other hero
            if (playerOne.isInMagnetWaveBounds(p2Bound.Center, p2radiusPoint, mDrawHelper))
            {
                //TODO: use float speed 
                Vector3 newP2Velocity = new Vector3();
                newP2Velocity.X += speed * -.04f * (p2Bound.Center.X -
                (playerOne.GetHeroBound(mDrawHelper).Center.X));
                newP2Velocity.Y += speed * -.04f * (p2Bound.Center.Y -
                (playerOne.GetHeroBound(mDrawHelper).Center.Y));

                playerTwo.setVelocityDeltaModel(newP2Velocity);
            }

        }

        public void P2ActivatePullMagnet(float speed)
        {
            Vector3 center;
            Vector3 point;
            // MAGNET TO RESOURCE HANDLING
            for (int i = 0; i < resourceList.Count; i++)
            {
                center = resourceList[i].getObjectBound().Center;
                point = new Vector3(center.X + resourceList[i].getObjectBound().Radius,
                    center.Y + resourceList[i].getObjectBound().Radius, 0);
                if (playerTwo.isInHeroCoverageArea(center))
                {
                    if (playerTwo.isInMagnetWaveBounds(center, point, mDrawHelper))
                    {
                        P2currentObj = resourceList[i];
                        break;
                    }
                }
            } // END OF MAGNET TO RESOURCE HANDLING
            // P2 MAGNET TO P1 BODY HANDLING
            // this will first check if the magnet bound is touching the other hero.
            // if it is, we will take the center point of p1 and p2, and change p2's
            // velocity so that he moves towards p1.
            UWB_BoundingCircle p1Bound = (UWB_BoundingCircle)playerOne.GetHeroBound(mDrawHelper);
            Vector3 p1radiusPoint = new Vector3(p1Bound.Center.X, p1Bound.GetCenter().Y +
                p1Bound.Radius, 0);
            // if player one's magnet is touching other hero
            if (playerTwo.isInMagnetWaveBounds(p1Bound.Center, p1radiusPoint, mDrawHelper))
            {
                //TODO: use float speed 
                Vector3 newP1Velocity = new Vector3();
                newP1Velocity.X += speed * -.04f * (p1Bound.Center.X -
                (playerTwo.GetHeroBound(mDrawHelper).Center.X));
                newP1Velocity.Y += speed * -.04f * (p1Bound.Center.Y -
                (playerTwo.GetHeroBound(mDrawHelper).Center.Y));

                playerOne.setVelocityDeltaModel(newP1Velocity);
            }

        }


        public void P1MagnetPull(float speed)
        {
            Vector3 center;
            if (P1currentObj != null)
            {
                center = P1currentObj.getObjectBound().Center;
                Vector3 front = playerOne.GetFront(mDrawHelper);
                Vector3 direction = Vector3.Normalize(front - center);

                if (!P1currentObj.isBeingPossesed)
                {
                    P1currentObj.setVelocity(direction * speed);

                    if (Vector3.Distance(center, front) < 0.1f)
                        P1currentObj.isBeingPossesed = true;
                }
                else
                {
                    P1currentObj.Reposition(front);
                    //playerOne.setVelocityDeltaModel(-direction * speed);
                }
            }
        }
        public void P2MagnetPull(float speed)
        {
            Vector3 center;
            if (P2currentObj != null)
            {
                center = P2currentObj.getObjectBound().Center;
                Vector3 front = playerTwo.GetFront(mDrawHelper);
                Vector3 direction = Vector3.Normalize(front - center);

                if (!P2currentObj.isBeingPossesed)
                {
                    P2currentObj.setVelocity(direction * speed);

                    if (Vector3.Distance(center, front) < 0.1f)
                        P2currentObj.isBeingPossesed = true;
                }
                else
                {
                    P2currentObj.Reposition(front);
                    //playerOne.setVelocityDeltaModel(-direction * speed);
                }
            }
        }

        public void P1PullMagnetHandler(float speed)
        {
            if (P1currentObj == null)
            {
                P1ActivatePullMagnet(speed);
            }
            else if (P1currentObj != null)
            {
                // manipulate the current obj
                P1MagnetPull(speed);
            }
        }
        public void P2PullMagnetHandler(float speed)
        {
            if (P2currentObj == null)
            {
                P2ActivatePullMagnet(speed);
            }
            else if (P2currentObj != null)
            {
                // manipulate the current obj
                P2MagnetPull(speed);
            }
        }

        // takes care of pushing the other hero if it needs to
        public void P1ActivatePushMagnet(float speed)
        {
            // P1 MAGNET TO P2 BODY HANDLING
            // this will first check if the magnet bound is touching the other hero.
            // if it is, we will take the center point of p1 and p2, and change p2's
            // velocity so that he moves away from p1.
            UWB_BoundingCircle p2Bound = (UWB_BoundingCircle)playerTwo.GetHeroBound(mDrawHelper);
            Vector3 p2radiusPoint = new Vector3(p2Bound.Center.X, p2Bound.GetCenter().Y +
                p2Bound.Radius, 0);
            // if player one's magnet is touching other hero
            if (playerOne.isInMagnetWaveBounds(p2Bound.Center, p2radiusPoint, mDrawHelper))
            {
                //TODO: use float speed 
                Vector3 newP2Velocity = new Vector3();
                newP2Velocity.X += speed * .04f * (p2Bound.Center.X -
                (playerOne.GetHeroBound(mDrawHelper).Center.X));
                newP2Velocity.Y += speed * .04f * (p2Bound.Center.Y -
                (playerOne.GetHeroBound(mDrawHelper).Center.Y));
                playerTwo.setVelocityDeltaModel(newP2Velocity);
            }
        }

        // takes care of pushing the other hero if it needs to
        public void P2ActivatePushMagnet(float speed)
        {
            // P1 MAGNET TO P2 BODY HANDLING
            // this will first check if the magnet bound is touching the other hero.
            // if it is, we will take the center point of p1 and p2, and change p2's
            // velocity so that he moves away from p1.
            UWB_BoundingCircle p1Bound = (UWB_BoundingCircle)playerOne.GetHeroBound(mDrawHelper);
            Vector3 p1radiusPoint = new Vector3(p1Bound.Center.X, p1Bound.GetCenter().Y +
                p1Bound.Radius, 0);
            // if player one's magnet is touching other hero
            if (playerTwo.isInMagnetWaveBounds(p1Bound.Center, p1radiusPoint, mDrawHelper))
            {
                //TODO: use float speed 
                Vector3 newP1Velocity = new Vector3();
                newP1Velocity.X += speed * .04f * (p1Bound.Center.X -
                (playerTwo.GetHeroBound(mDrawHelper).Center.X));
                newP1Velocity.Y += speed * .04f * (p1Bound.Center.Y -
                (playerTwo.GetHeroBound(mDrawHelper).Center.Y));
                playerOne.setVelocityDeltaModel(newP1Velocity);
            }
        }

        // will check both push power and check if an object is currently held.
        // if one is held it will shoot out quickly
        public void P1PushMagnetHandler(float speedPush)
        {
            // if were holding an ojbect shoot it out.
            if (P1currentObj != null)
            {   // only fire if the projectile is being held
                if (P1currentObj.isBeingPossesed == true)
                {
                    // if its a heatseeker thats not fired or if its a straight shot
                    if (!P1currentObj.isFired || P1currentObj.typeOfResource == eResourceType.straightShot)
                    {
                        Vector3 initDir = new Vector3();
                        // reset hero direction within hero itself
                        playerOne.isCollidingWithMagnet(initDir, initDir, mDrawHelper);
                        // get newly updated hero direction
                        initDir = playerOne.GetHeroDirection();
                        // NEW ENTRY
                        if (!P1currentObj.isFired && P1currentObj.typeOfResource == eResourceType.heatSeeker)
                            soundManager.Play("shottBigProjectile");
                        if (!P1currentObj.isFired && P1currentObj.typeOfResource == eResourceType.straightShot)
                            soundManager.Play("shootSmallProjectile");
                        // END NEW ENTRY
                        // shoot projectile the direction magnet is facing
                        P1currentObj.shootProjectile(initDir * speedPush);
                    }
                }
                else
                    P1LetGoOfObject();
            }
            //else if (P2currentObj.typeOfResource == eResourceType.straightShot)
            P1ActivatePushMagnet(speedPush);

            // call P1ActivatePushMagnet to check if there is a collision with hero.
            // if there is push him.
        }

        public void P2PushMagnetHandler(float speedPush)
        {
            // if were holding an ojbect shoot it out.
            if (P2currentObj != null)
            {   // only fire if the projectile is being held
                if (P2currentObj.isBeingPossesed == true)
                {
                    // if its a heatseeker thats not fired or if its a straight shot
                    if (!P2currentObj.isFired || P2currentObj.typeOfResource == eResourceType.straightShot)
                    {
                        Vector3 initDir = new Vector3();
                        // reset hero direction within hero itself
                        playerTwo.isCollidingWithMagnet(initDir, initDir, mDrawHelper);
                        // get newly updated hero direction
                        initDir = playerTwo.GetHeroDirection();
                        // NEW ENTRY
                        if (!P2currentObj.isFired && P2currentObj.typeOfResource == eResourceType.heatSeeker)
                            soundManager.Play("shottBigProjectile");
                        if (!P2currentObj.isFired && P2currentObj.typeOfResource == eResourceType.straightShot)
                            soundManager.Play("shootSmallProjectile");
                        // END NEW ENTRY
                        // shoot projectile the direction magnet is facing
                        P2currentObj.shootProjectile(initDir * speedPush);
                    }
                }
                else
                    P2LetGoOfObject();
            }
            //else if (P2currentObj.typeOfResource == eResourceType.straightShot)
            P2ActivatePushMagnet(speedPush);

            // call P1ActivatePushMagnet to check if there is a collision with hero.
            // if there is push him.
        }

        public void P1LetGoOfObject()
        {
            if (P1currentObj != null)
            {
                P1currentObj.setVelocity(new Vector3(0, 0, 0));
                P1currentObj.isBeingPossesed = false;
                P1currentObj = null;
            }
        }
        public void P2LetGoOfObject()
        {
            if (P2currentObj != null)
            {
                P2currentObj.setVelocity(new Vector3(0, 0, 0));
                P2currentObj.isBeingPossesed = false;
                P2currentObj = null;
            }
        }

        #endregion

        #region Misc Controls
        public bool pause()
        {
            m_pause = m_pause == false ? true : false;
            return m_pause;
        }

        public bool isPaused()
        {
            return m_pause;
        }

        public void setHealthLvl(int x)
        {
            m_Progressbar.CurrentValue += x;
        }

        public void ChangeZoomLevel(int x)
        {
            // X = 0 -> p1, x = 1 -> p2
            if (x == 0)
                m_P1zoomlvl = m_P1zoomlvl == 1.0f ? 1.5f : 1.0f;
            else
                m_P2zoomlvl = m_P2zoomlvl == 1.0f ? 1.5f : 1.0f;
        }

        public void Reset()
        {
            if (!m_pause)   // if were not paused then you cant restart.
                return;

            m_stage = 1;
            resourceList.Clear();
            resourceGenerator = new ResrouceManager();
            m_Obstacles.Clear();
            m_Obstacles.Initialize(m_stage);
            ResetP1Position();
            ResetP1Stat();
            ResetP2Position();
            ResetP2Stat();

            m_pause = false;
        }

        public void ResetP1Position()
        {
            P1currentObj = null;
            playerOne = new Hero(1);

            UWB_XformInfo xf = playerOne.getRootNode().getXformInfo();
            xf.SetPivot(new Vector3(86.7f, 23, 0));
            xf.SetTranslation(new Vector3(-67.2f, 43, 0));
            xf.SetRotationInDegrees(-90);
            playerOne.getRootNode().setXformInfo(xf);

        }

        public void ResetP1Stat()
        {
            m_P1life = 3;
            m_Progressbar.CurrentValue = 50;
        }

        public void ResetP2Position()
        {
            P2currentObj = null;
            playerTwo = new Hero(2);

            UWB_XformInfo xf = playerTwo.getRootNode().getXformInfo();
            xf.SetPivot(new Vector3(86.7f, 23, 0));
            xf.SetTranslation(new Vector3(338.3f, 43, 0));
            xf.SetRotationInDegrees(90);
            playerTwo.getRootNode().setXformInfo(xf);
        }

        public void ResetP2Stat()
        {
            m_P2life = 3;
            m_Progressbar.CurrentValue = 50;
        }

        #endregion

        #region World Object Interaction
        public void HoleCollisionCheck()
        {
            //if(m_Obstacles.inAnObstacle(playerOne.GetHeroBound(mDrawHelper)))
            for (int i = 0; i < m_Obstacles.getObstacles().Count; i++)
            {
                if (m_Obstacles.getObstacles()[i].isInObstacle(playerOne.GetHeroBound(mDrawHelper)))
                {
                    if (!soundManager.IsPlaying("warpCollision"))
                        soundManager.Play("warpCollision"); // NEW ENTRY
                    m_P1GravityCenter = m_Obstacles.getObstacles()[i].getObjectBound().Center;

                    FlushP1();
                    //ResetP1Position();
                    m_Progressbar.CurrentValue = 50;
                    
                }
                if (m_Obstacles.getObstacles()[i].isInObstacle(playerTwo.GetHeroBound(mDrawHelper)))
                {
                    soundManager.Play("warpCollision"); // NEW ENTRY
                    m_P2GravityCenter = m_Obstacles.getObstacles()[i].getObjectBound().Center;

                    FlushP2();
                    m_Progressbar.CurrentValue = 50;               
                }
            }
            // is resource colliding with obstacle
            for (int i = 0; i < resourceList.Count; i++)
            {
                if (m_Obstacles.inAnObstacle(resourceList[i].getObjectBound()))
                {
                    resourceGenerator.deleteResource(); // decrement counter
                    resourceList.RemoveAt(i);
                    break;
                }
            }
        }

        public void FlushP1()
        {
            if (playerOne.getRootNode().getXformInfo().GetScale().X > 0.01f)
            {

                UWB_XformInfo xf = playerOne.getRootNode().getXformInfo();

                xf.SetPivot(m_P1GravityCenter);
                xf.SetTranslation(new Vector3(0, 0, 0));
                xf.SetRotationInDegrees(playerOne.getRootNode().getXformInfo().GetRotatioinInDegrees() +
                    (float)Math.Sin(1) * 0.15f * 20);
                xf.SetScale(playerOne.getRootNode().getXformInfo().GetScale() * 0.99f);
                playerOne.getRootNode().setXformInfo(xf);
            }
            else
            {
                ResetP1Position();
                m_P1life -= 1;
            }
        }

        public void FlushP2()
        {
            if (playerTwo.getRootNode().getXformInfo().GetScale().X > 0.01f)
            {

                UWB_XformInfo xf = playerTwo.getRootNode().getXformInfo();

                xf.SetPivot(m_P2GravityCenter);
                xf.SetTranslation(new Vector3(0, 0, 0));
                xf.SetRotationInDegrees(playerTwo.getRootNode().getXformInfo().GetRotatioinInDegrees() +
                    (float)Math.Sin(1) * 0.15f * 20);
                xf.SetScale(playerTwo.getRootNode().getXformInfo().GetScale() * 0.99f);
                playerTwo.getRootNode().setXformInfo(xf);
            }
            else
            {
                ResetP2Position();
                m_P2life -= 1;
            }
        }
        #endregion

        #region Player Stat Management
        public void UpdatePlayerCondition()
        {
            if (m_Progressbar.CurrentValue == 0 )
            {
                if (playerOne.getRootNode().getXformInfo().GetScale().X < 10.0f)
                    playerOne.DeathAnimation(0.50f);
                else
                {
                    m_Progressbar.CurrentValue = 50;
                    ResetP1Position();
                    m_P1life -= 1;
                    soundManager.Play("heroExplosion");
                }
                
            }
            else if( m_Progressbar.CurrentValue == 100 )
            {
                if (playerTwo.getRootNode().getXformInfo().GetScale().X < 10.0f)
                    playerTwo.DeathAnimation(0.50f);
                else
                {
                    m_Progressbar.CurrentValue = 50;
                    ResetP2Position();
                    m_P2life -= 1;
                    soundManager.Play("heroExplosion");
                }
            }

        }

        public void CheckWinningCondition()
        {
            if (m_P1life <= 0)
            {
                ResetP1Stat();
                ResetP2Stat();
                isP1Gameover = true;
                AdvanceGame();
            }
            else if (m_P2life <= 0)
            {
                ResetP1Stat();
                ResetP2Stat();
                isP2Gameover = true;
                AdvanceGame();
            }

        }

        public void AdvanceGame()
        {
            m_stage += 1;
            switch (m_stage)
            {
                case 2:
                    {
                        background.SetTextureFilename("background2");
                        break;
                    }
                default:
                    {
                        background.SetTextureFilename("background3");
                        break;
                    }
            }
            ResetP1Position();
            ResetP2Position();
            m_Obstacles.Initialize(m_stage);
        }

        public int GetP1Stat()
        {
            return m_P1life;
        }

        public int GetP2Stat()
        {
            return m_P2life;
        }

        #endregion
    }
}