using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace MagGame
{
    public enum eResourceType { straightShot, heatSeeker };

    public class ResourceObject : WorldObject
    {
        private eResourceType resourceType;
        public eResourceType typeOfResource
        {
            get { return resourceType; }
        }
        // TODO: need rectangle for different resource types
        private UWB_PrimitiveCircle myPrimitive;
        private Random myRand;
        private Vector3 myVelocity;
        private bool isShot;
        private bool isPossesed;
        private Vector3 maxWorldCorner;
        private float degree;
        public bool isFired
        {
            get { return isShot; }
        }
        private List<UWB_PrimitivePoint> particles;
        UWB_XformInfo xf;


        // creating resource object by: giving it a random position within world
        // bounds (consturctor assume minumum world bound is 0,0), the resource type
        // is randomly decided.
        public ResourceObject(Vector3 max)
        {
            xf = new UWB_XformInfo();
            degree = 0f;
            maxWorldCorner = max;
            particles = new List<UWB_PrimitivePoint>();
            myRand = new Random();
            // TODO: made a random chance for it to be a coin or a particular
            // type of powerup.
            if (myRand.Next(1, 5) == 1)  // 1 in 5 chance of powerup
                resourceType = eResourceType.heatSeeker;
            else
                resourceType = eResourceType.straightShot;

            // TODO: depending on the resource type, the size/texture will change
            myPrimitive = new UWB_PrimitiveCircle();

            // TODO: resrouce allocation should avoid positioning resource on a hole

            // half of the coins will appear very near the center
            int distFromCenter = myRand.Next(0, 2);
            float posX = 0f;
            float posY = 0f;
            if (distFromCenter == 1)    // anywhere from center
            {
                posX = max.X / 2;
                posX += -(max.X / 4f) + myRand.Next(0, (int)(max.X / 2f));
                posY = (float)myRand.NextDouble() * max.Y;
            }

            else    // coin appears very near the center
            {
                posX = max.X * 4 / 10;
                posX += myRand.Next(0, (int)(max.X / 5f));
                posY = max.Y / 2;
                posY += -(max.Y / 4f) + myRand.Next(0, (int)(max.Y / 2f));
            }
            /* float posY = max.Y / 2;
             posY += -(max.Y / 4f) + myRand.Next(0, (int)(max.Y / 2f));
             float posX = (float)myRand.NextDouble() * max.X;
             if (posX < max.X / 4f)
                 posX += max.X / 4f;
             if (posX > max.X / 3f)
                 posX -= max.X / 4f;
              * */
            float myRadius = 0f;
            if (resourceType == eResourceType.straightShot)
                myRadius = .7f;
            else   // if its a powerup, it is a bigger circle
                myRadius = 1.4f;

            myCenter = new Vector3(posX, posY, 0);
            myPrimitive.setRadius(myRadius);
            myPrimitive.setCenter(myCenter);
            myPrimitive.EnableBlending(true);
            myPrimitive.EnableTexturing(true);
            if (resourceType == eResourceType.heatSeeker)
                myPrimitive.SetTextureFilename("tire");
            if (resourceType == eResourceType.straightShot)
                myPrimitive.SetTextureFilename("basicshot");
            myPrimitive.setShadeMode(eShadeMode.smFlat);
            myPrimitive.setFlatColor(Color.DarkOrange);
            // bound is bigger so it can be visible TODO: remove .5f
            myBound = new UWB_BoundingCircle(myCenter, myRadius + .5f);
            isShot = false;
            isPossesed = false;
        }

        public void update(float elapsedSeconds)
        {
            if (isShot && resourceType == eResourceType.straightShot)
            {
                xf.SetRotationInRadians(xf.GetRotationInRadians() + (float)Math.Sin(1) * elapsedSeconds*10);
            }
            myVelocity.X = MathHelper.Clamp(myVelocity.X, -10, 10);
            myVelocity.Y = MathHelper.Clamp(myVelocity.Y, -10, 10);

            myPrimitive.setVelocity(myVelocity);
            myPrimitive.update(elapsedSeconds);
            myBound.Center = myPrimitive.getLocation();
            myBound.Radius = myPrimitive.getRadius();
            particleEffect(elapsedSeconds);

        }

        // create x*speed particles every second. particle velocity is opposite of
        // projectile velocity, with reduced magnitude, and slight alteration.
        // decrease size of each point each second.
        // if a particle reaches its min size, delete it.
        public void particleEffect(float elapsedSeconds)
        {
            float size = 5f; // initial size of particles
            int numNewParticles = (int)(elapsedSeconds * myVelocity.Length() *
                UWB_Utility.UWB_RandomNumber(2f, 3.5f));
            // add in the number of particle we need.
            for (int i = 0; i < numNewParticles; i++)
            {
                float x = UWB_Utility.UWB_RandomNumber(0.1f, 0.4f);
                float y = UWB_Utility.UWB_RandomNumber(0.1f, 0.4f);
                // note myVelocity may not have been clamped
                Vector3 pointVel = new Vector3(myVelocity.X * x, myVelocity.Y * y, 0f);
                UWB_PrimitivePoint tiny = new UWB_PrimitivePoint();
                tiny.setLocation(myPrimitive.getLocation());
                tiny.setPointSize(size);
                tiny.setVelocity(pointVel);
                tiny.EnableTexturing(true);
                tiny.SetTextureFilename("HealthBar");
                particles.Add(tiny);
            }
            // update and remove any particles that are too small
            for (int i = 0; i < particles.Count; i++)
            {
                particles[i].update(elapsedSeconds);
                // lose 1f in size per second
                particles[i].setPointSize(particles[i].getPointSize() - 1f * elapsedSeconds);
                if (particles[i].getPointSize() <= 3f)
                {
                    particles.RemoveAt(i);
                }
            }
        }



        public void setVelocity(Vector3 v)
        {
            myVelocity = v;
        }

        public void Reposition(Vector3 point)
        {
            myPrimitive.setCenter(point);
        }

        public Vector3 getVelocity()
        {
            return myPrimitive.getVelocity();
        }

        public void resourceDraw(UWB_DrawHelper helper)
        {
            if (isShot && resourceType == eResourceType.straightShot)
            {
                helper.PushModelTransform();
            }

            myPrimitive.draw(eLevelofDetail.lodHigh, helper);
            helper.setFillMode(eFillMode.fmWireframe);
            helper.setColor2(Color.Red);
            myBound.Draw(helper);
            // draw particle effect
            foreach (UWB_PrimitivePoint tiny in particles)
                tiny.draw(eLevelofDetail.lodHigh, helper);

            if (isShot && resourceType == eResourceType.straightShot)
            {
                helper.PopModelTransform();
            }
        }

        public Vector3 getPosition()
        {
            return myPrimitive.getLocation();
        }

        public void shootProjectile(Vector3 initVel)
        {
            myVelocity = initVel * 10f;
            isShot = true;
        }

        public void updateHomingPosition(Vector3 home)
        {
            Vector3 deltaVel = new Vector3();
            deltaVel = home - myPrimitive.getLocation();
            myVelocity.X = myVelocity.X + deltaVel.X * .01f;
            myVelocity.Y = myVelocity.Y + deltaVel.Y * .01f;
        }

        public override UWB_BoundingCircle getObjectBound()
        {
            return myBound;
        }

        public bool isBeingPossesed
        {
            get
            {
                return isPossesed;
            }
            set
            {
                isPossesed = value;
            }
        }
    }
}