using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace MagGame
{
    public class Hero
    {
        public UWB_SceneNode m_sRoot;
        public UWB_PrimitiveList netPrimitive;
        private UWB_PrimitiveCircle m_magnetcollisionLeft;
        private UWB_PrimitiveCircle m_magnetcollisionRight;
        private UWB_PrimitiveCircle m_magnetcollisionBtm;
        private UWB_PrimitivePoint m_front;

        private UWB_PrimitiveCircle m_magwavebtm;
        private UWB_PrimitiveCircle m_magwavemdl;
        private UWB_PrimitiveCircle m_magwavetop;

        private UWB_PrimitiveCircle movingCircle;

        // bound that covers the whole hero
        private UWB_BoundingCircle HeroBound;
        private UWB_BoundingCircle BigHeroBound;
        private UWB_BoundingCircle MagnetBound;
        private Vector3 HeroDirection;

        private UWB_BoundingBox m_MagnetWaveBox;
        private List<UWB_PrimitivePoint> m_points;
        private Random m_rand;
        private bool animateWave;
        private int playernum;
        // Constructor
        public Hero(int pn)
        {
            playernum = pn;
            ConstructHero();
            magnetFuel = 1f;   // 10f is a full tank

            HeroBound = new UWB_BoundingCircle();
            BigHeroBound = new UWB_BoundingCircle();
            MagnetBound = new UWB_BoundingCircle();

            Vector3 corner1 = new Vector3(83.7f, 35, 0);
            Vector3 corner2 = new Vector3(89.7f, 45, 0);
            m_MagnetWaveBox = new UWB_BoundingBox();
            m_MagnetWaveBox.SetCorners(corner1, corner2);

            m_points = new List<UWB_PrimitivePoint>();
            animateWave = false;
        }

        public UWB_SceneNode getRootNode()
        {
            return m_sRoot;
        }

        private void ConstructHero()
        {
            vChangeByModel = new Vector3();
            // Move everything by 82.2 in the X direction and 17 in the Y direction
            // Basic varibles need for constructing our hero
            Vector3 min = new Vector3(0, 0, 0);
            Vector3 max = new Vector3(0, 0, 0);
            Vector3 center = new Vector3(0, 0, 0);
            float radius = 0.0f;
            UWB_SceneNode childNode = new UWB_SceneNode("childNode_PlaceHolder");
            UWB_SceneNode subChildNode = new UWB_SceneNode("subChildNode_PlaceHolder");

            // Construct the body of our hero
            m_sRoot = new UWB_SceneNode("Body");
            UWB_PrimitiveRectangle body = new UWB_PrimitiveRectangle();
            min = new Vector3(84.2f, 17.0f, 0.0f);
            max = new Vector3(89.2f, 29.0f, 0.0f);
            body.SetCorners(min, max);

            body.EnableBlending(true);
            body.EnableTexturing(true);
            if (playernum == 1)
                body.SetTextureFilename("bodyred");
            else
                body.SetTextureFilename("bodyblue");
            body.setShadeMode(eShadeMode.smFlat);
            body.setFlatColor(Color.White);

            m_sRoot.setPrimitive(body);
            UWB_XformInfo xf = m_sRoot.getXformInfo();
            Vector3 pivPos = Vector3.Divide((min + max), 2);
            m_sRoot.originalPivot = pivPos;
            xf.SetPivot(pivPos);
            m_sRoot.setXformInfo(xf);

            // Construct the bottom left axle of our hero
            childNode = new UWB_SceneNode("axle_btm_left");
            UWB_PrimitiveRectangle axle_btm_left = new UWB_PrimitiveRectangle();
            min = new Vector3(82.2f, 19.0f, 0.0f);
            max = new Vector3(84.2f, 20.0f, 0.0f);
            axle_btm_left.SetCorners(min, max);

            axle_btm_left.EnableBlending(true);
            axle_btm_left.EnableTexturing(true);
            axle_btm_left.SetTextureFilename("carbon");
            axle_btm_left.setShadeMode(eShadeMode.smFlat);
            axle_btm_left.setFlatColor(Color.White);

            childNode.setPrimitive(axle_btm_left);

            xf = new UWB_XformInfo();
            xf = childNode.getXformInfo();
            pivPos = Vector3.Divide((min + max), 2);
            pivPos.X = max.X;
            childNode.originalPivot = pivPos;
            xf.SetPivot(pivPos);
            childNode.setXformInfo(xf);

            m_sRoot.insertChildNode(childNode);


            {
                // Construct the bottom tire for this axle
                subChildNode = new UWB_SceneNode("axle_btm_left_tire_btm");
                UWB_PrimitiveCircle axle_btm_left_tire_btm = new UWB_PrimitiveCircle();
                center = new Vector3(82.7f, 19.0f, 0.0f);
                radius = 0.5f;
                axle_btm_left_tire_btm.setCenter(center);
                axle_btm_left_tire_btm.setRadius(radius);

                axle_btm_left_tire_btm.EnableBlending(true);
                axle_btm_left_tire_btm.EnableTexturing(true);
                axle_btm_left_tire_btm.SetTextureFilename("yellowtire");
                axle_btm_left_tire_btm.setShadeMode(eShadeMode.smFlat);
                axle_btm_left_tire_btm.setFlatColor(Color.White);

                subChildNode.setPrimitive(axle_btm_left_tire_btm);
                xf = new UWB_XformInfo();
                xf = subChildNode.getXformInfo();
                subChildNode.originalPivot = pivPos;
                xf.SetPivot(new Vector3(82.7f, 19.5f, 0));
                subChildNode.setXformInfo(xf);
                childNode.insertChildNode(subChildNode);

                // Construct the top tire for this axle
                subChildNode = new UWB_SceneNode("axle_btm_left_tire_top");
                UWB_PrimitiveCircle axle_btm_left_tire_top = new UWB_PrimitiveCircle();
                center = new Vector3(82.7f, 20.0f, 0.0f);
                radius = 0.5f;
                axle_btm_left_tire_top.setCenter(center);
                axle_btm_left_tire_top.setRadius(radius);

                axle_btm_left_tire_top.EnableBlending(true);
                axle_btm_left_tire_top.EnableTexturing(true);
                axle_btm_left_tire_top.SetTextureFilename("yellowtire");
                axle_btm_left_tire_top.setShadeMode(eShadeMode.smFlat);
                axle_btm_left_tire_top.setFlatColor(Color.White);

                subChildNode.setPrimitive(axle_btm_left_tire_top);
                xf = new UWB_XformInfo();
                xf = subChildNode.getXformInfo();
                subChildNode.originalPivot = pivPos;
                xf.SetPivot(new Vector3(82.7f, 19.5f, 0));
                subChildNode.setXformInfo(xf);
                childNode.insertChildNode(subChildNode);
            }

            // Construct the middle left axle of our hero
            childNode = new UWB_SceneNode("axle_mid_left");
            UWB_PrimitiveRectangle axle_mid_left = new UWB_PrimitiveRectangle();
            min = new Vector3(82.2f, 22.0f, 0.0f);
            max = new Vector3(84.2f, 23.0f, 0.0f);
            axle_mid_left.SetCorners(min, max);

            axle_mid_left.EnableBlending(true);
            axle_mid_left.EnableTexturing(true);
            axle_mid_left.SetTextureFilename("carbon");
            axle_mid_left.setShadeMode(eShadeMode.smFlat);
            axle_mid_left.setFlatColor(Color.White);

            childNode.setPrimitive(axle_mid_left);
            xf = new UWB_XformInfo();
            xf = childNode.getXformInfo();
            pivPos = Vector3.Divide((min + max), 2);
            pivPos.X = max.X;
            childNode.originalPivot = pivPos;
            subChildNode.originalPivot = pivPos;
            xf.SetPivot(pivPos);
            childNode.setXformInfo(xf);
            m_sRoot.insertChildNode(childNode);

            {

                // Construct the top tire for this axle
                subChildNode = new UWB_SceneNode("axle_mid_left_tire_top");
                UWB_PrimitiveCircle axle_mid_left_tire_top = new UWB_PrimitiveCircle();
                center = new Vector3(82.7f, 23.0f, 0.0f);
                radius = 0.5f;
                axle_mid_left_tire_top.setCenter(center);
                axle_mid_left_tire_top.setRadius(radius);

                axle_mid_left_tire_top.EnableBlending(true);
                axle_mid_left_tire_top.EnableTexturing(true);
                axle_mid_left_tire_top.SetTextureFilename("yellowtire");
                axle_mid_left_tire_top.setShadeMode(eShadeMode.smFlat);
                axle_mid_left_tire_top.setFlatColor(Color.White);

                subChildNode.setPrimitive(axle_mid_left_tire_top);
                xf = new UWB_XformInfo();
                xf = subChildNode.getXformInfo();
                subChildNode.originalPivot = pivPos;
                xf.SetPivot(new Vector3(82.7f, 22.5f, 0.0f));
                subChildNode.setXformInfo(xf);
                childNode.insertChildNode(subChildNode);

                // Construct the bottom tire for this axle
                subChildNode = new UWB_SceneNode("axle_mid_left_tire_btm");
                UWB_PrimitiveCircle axle_mid_left_tire_btm = new UWB_PrimitiveCircle();
                center = new Vector3(82.7f, 22.0f, 0.0f);
                radius = 0.5f;
                axle_mid_left_tire_btm.setCenter(center);
                axle_mid_left_tire_btm.setRadius(radius);


                axle_mid_left_tire_btm.EnableBlending(true);
                axle_mid_left_tire_btm.EnableTexturing(true);
                axle_mid_left_tire_btm.SetTextureFilename("yellowtire");
                axle_mid_left_tire_btm.setShadeMode(eShadeMode.smFlat);
                axle_mid_left_tire_btm.setFlatColor(Color.White);

                subChildNode.setPrimitive(axle_mid_left_tire_btm);
                xf = new UWB_XformInfo();
                xf = subChildNode.getXformInfo();
                subChildNode.originalPivot = pivPos;
                xf.SetPivot(new Vector3(82.7f, 22.5f, 0.0f));
                subChildNode.setXformInfo(xf);
                childNode.insertChildNode(subChildNode);

            }

            // Construct the top left axle of our hero
            childNode = new UWB_SceneNode("axle_top_left");
            UWB_PrimitiveRectangle axle_top_left = new UWB_PrimitiveRectangle();
            min = new Vector3(82.2f, 25.0f, 0.0f);
            max = new Vector3(84.2f, 26.0f, 0.0f);
            axle_top_left.SetCorners(min, max);

            axle_top_left.EnableBlending(true);
            axle_top_left.EnableTexturing(true);
            axle_top_left.SetTextureFilename("carbon");
            axle_top_left.setShadeMode(eShadeMode.smFlat);
            axle_top_left.setFlatColor(Color.White);

            childNode.setPrimitive(axle_top_left);
            xf = new UWB_XformInfo();
            xf = childNode.getXformInfo();
            pivPos = Vector3.Divide((min + max), 2);
            pivPos.X = max.X;
            childNode.originalPivot = pivPos;
            xf.SetPivot(pivPos);
            childNode.setXformInfo(xf);
            m_sRoot.insertChildNode(childNode);

            {
                // Construct the top tire for this axle
                subChildNode = new UWB_SceneNode("axle_top_left_tire_top");
                UWB_PrimitiveCircle axle_top_left_tire_top = new UWB_PrimitiveCircle();
                center = new Vector3(82.7f, 26.0f, 0.0f);
                radius = 0.5f;
                axle_top_left_tire_top.setCenter(center);
                axle_top_left_tire_top.setRadius(radius);

                axle_top_left_tire_top.EnableBlending(true);
                axle_top_left_tire_top.EnableTexturing(true);
                axle_top_left_tire_top.SetTextureFilename("yellowtire");
                axle_top_left_tire_top.setShadeMode(eShadeMode.smFlat);
                axle_top_left_tire_top.setFlatColor(Color.White);

                subChildNode.setPrimitive(axle_top_left_tire_top);
                xf = new UWB_XformInfo();
                xf = subChildNode.getXformInfo();
                subChildNode.originalPivot = pivPos;
                xf.SetPivot(new Vector3(82.7f, 25.5f, 0.0f));
                subChildNode.setXformInfo(xf);
                childNode.insertChildNode(subChildNode);

                // Construct the bottom tire for this axle
                subChildNode = new UWB_SceneNode("axle_top_left_tire_btm");
                UWB_PrimitiveCircle axle_top_left_tire_btm = new UWB_PrimitiveCircle();
                center = new Vector3(82.7f, 25.0f, 0.0f);
                radius = 0.5f;
                axle_top_left_tire_btm.setCenter(center);
                axle_top_left_tire_btm.setRadius(radius);

                axle_top_left_tire_btm.EnableBlending(true);
                axle_top_left_tire_btm.EnableTexturing(true);
                axle_top_left_tire_btm.SetTextureFilename("yellowtire");
                axle_top_left_tire_btm.setShadeMode(eShadeMode.smFlat);
                axle_top_left_tire_btm.setFlatColor(Color.White);

                subChildNode.setPrimitive(axle_top_left_tire_btm);
                xf = new UWB_XformInfo();
                xf = subChildNode.getXformInfo();
                subChildNode.originalPivot = pivPos;
                xf.SetPivot(new Vector3(82.7f, 25.5f, 0.0f));
                subChildNode.setXformInfo(xf);
                childNode.insertChildNode(subChildNode);


            }

            // Construct the bottom right axle of our hero
            childNode = new UWB_SceneNode("axle_btm_right");
            UWB_PrimitiveRectangle axle_btm_right = new UWB_PrimitiveRectangle();
            min = new Vector3(89.2f, 19.0f, 0.0f);
            max = new Vector3(91.2f, 20.0f, 0.0f);
            axle_btm_right.SetCorners(min, max);

            axle_btm_right.EnableBlending(true);
            axle_btm_right.EnableTexturing(true);
            axle_btm_right.SetTextureFilename("carbon");
            axle_btm_right.setShadeMode(eShadeMode.smFlat);
            axle_btm_right.setFlatColor(Color.White);

            childNode.setPrimitive(axle_btm_right);
            xf = new UWB_XformInfo();
            xf = childNode.getXformInfo();
            pivPos = Vector3.Divide((min + max), 2);
            pivPos.X = min.X;
            childNode.originalPivot = pivPos;
            xf.SetPivot(pivPos);
            childNode.setXformInfo(xf);
            m_sRoot.insertChildNode(childNode);

            {
                // Construct the top tire for this axle
                subChildNode = new UWB_SceneNode("axle_btm_right_tire_top");
                UWB_PrimitiveCircle axle_btm_right_tire_top = new UWB_PrimitiveCircle();
                center = new Vector3(90.7f, 20.0f, 0.0f);
                radius = 0.5f;
                axle_btm_right_tire_top.setCenter(center);
                axle_btm_right_tire_top.setRadius(radius);

                axle_btm_right_tire_top.EnableBlending(true);
                axle_btm_right_tire_top.EnableTexturing(true);
                axle_btm_right_tire_top.SetTextureFilename("yellowtire");
                axle_btm_right_tire_top.setShadeMode(eShadeMode.smFlat);
                axle_btm_right_tire_top.setFlatColor(Color.White);

                subChildNode.setPrimitive(axle_btm_right_tire_top);
                xf = new UWB_XformInfo();
                xf = subChildNode.getXformInfo();
                subChildNode.originalPivot = pivPos;
                xf.SetPivot(new Vector3(90.7f, 19.5f, 0.0f));
                subChildNode.setXformInfo(xf);
                childNode.insertChildNode(subChildNode);

                // Construct the bottom tire for this axle
                subChildNode = new UWB_SceneNode("axle_btm_right_tire_btm");
                UWB_PrimitiveCircle axle_btm_right_tire_btm = new UWB_PrimitiveCircle();
                center = new Vector3(90.7f, 19.0f, 0.0f);
                radius = 0.5f;
                axle_btm_right_tire_btm.setCenter(center);
                axle_btm_right_tire_btm.setRadius(radius);

                axle_btm_right_tire_btm.EnableBlending(true);
                axle_btm_right_tire_btm.EnableTexturing(true);
                axle_btm_right_tire_btm.SetTextureFilename("yellowtire");
                axle_btm_right_tire_btm.setShadeMode(eShadeMode.smFlat);
                axle_btm_right_tire_btm.setFlatColor(Color.White);

                subChildNode.setPrimitive(axle_btm_right_tire_btm);
                xf = new UWB_XformInfo();
                xf = subChildNode.getXformInfo();
                subChildNode.originalPivot = pivPos;
                xf.SetPivot(new Vector3(90.7f, 19.5f, 0.0f));
                subChildNode.setXformInfo(xf);
                childNode.insertChildNode(subChildNode);
            }

            // Construct the middle right axle of our hero
            childNode = new UWB_SceneNode("axle_mid_right");
            UWB_PrimitiveRectangle axle_mid_right = new UWB_PrimitiveRectangle();
            min = new Vector3(89.2f, 22.0f, 0.0f);
            max = new Vector3(91.2f, 23.0f, 0.0f);
            axle_mid_right.SetCorners(min, max);

            axle_mid_right.EnableBlending(true);
            axle_mid_right.EnableTexturing(true);
            axle_mid_right.SetTextureFilename("carbon");
            axle_mid_right.setShadeMode(eShadeMode.smFlat);
            axle_mid_right.setFlatColor(Color.White);

            childNode.setPrimitive(axle_mid_right);
            xf = new UWB_XformInfo();
            xf = childNode.getXformInfo();
            pivPos = Vector3.Divide((min + max), 2);
            pivPos.X = min.X;
            childNode.originalPivot = pivPos;
            xf.SetPivot(pivPos);
            childNode.setXformInfo(xf);
            m_sRoot.insertChildNode(childNode);

            {
                // Construct the top tire for this axle
                subChildNode = new UWB_SceneNode("axle_mid_right_tire_top");
                UWB_PrimitiveCircle axle_mid_right_tire_top = new UWB_PrimitiveCircle();
                center = new Vector3(90.7f, 23.0f, 0.0f);
                radius = 0.5f;
                axle_mid_right_tire_top.setCenter(center);
                axle_mid_right_tire_top.setRadius(radius);

                axle_mid_right_tire_top.EnableBlending(true);
                axle_mid_right_tire_top.EnableTexturing(true);
                axle_mid_right_tire_top.SetTextureFilename("yellowtire");
                axle_mid_right_tire_top.setShadeMode(eShadeMode.smFlat);
                axle_mid_right_tire_top.setFlatColor(Color.White);

                subChildNode.setPrimitive(axle_mid_right_tire_top);
                xf = new UWB_XformInfo();
                xf = subChildNode.getXformInfo();
                subChildNode.originalPivot = pivPos;
                xf.SetPivot(new Vector3(90.7f, 22.5f, 0.0f));
                subChildNode.setXformInfo(xf);
                childNode.insertChildNode(subChildNode);

                // Construct the bottom tire for this axle
                subChildNode = new UWB_SceneNode("axle_mid_right_tire_btm");
                UWB_PrimitiveCircle axle_mid_right_tire_btm = new UWB_PrimitiveCircle();
                center = new Vector3(90.7f, 22.0f, 0.0f);
                radius = 0.5f;
                axle_mid_right_tire_btm.setCenter(center);
                axle_mid_right_tire_btm.setRadius(radius);

                axle_mid_right_tire_btm.EnableBlending(true);
                axle_mid_right_tire_btm.EnableTexturing(true);
                axle_mid_right_tire_btm.SetTextureFilename("yellowtire");
                axle_mid_right_tire_btm.setShadeMode(eShadeMode.smFlat);
                axle_mid_right_tire_btm.setFlatColor(Color.White);

                subChildNode.setPrimitive(axle_mid_right_tire_btm);
                xf = new UWB_XformInfo();
                xf = subChildNode.getXformInfo();
                subChildNode.originalPivot = pivPos;
                xf.SetPivot(new Vector3(90.7f, 22.5f, 0.0f));
                subChildNode.setXformInfo(xf);
                childNode.insertChildNode(subChildNode);
            }


            // Construct the top right axle of our hero
            childNode = new UWB_SceneNode("axle_top_right");
            UWB_PrimitiveRectangle axle_top_right = new UWB_PrimitiveRectangle();
            min = new Vector3(89.2f, 25.0f, 0.0f);
            max = new Vector3(91.2f, 26.0f, 0.0f);
            axle_top_right.SetCorners(min, max);

            axle_top_right.EnableBlending(true);
            axle_top_right.EnableTexturing(true);
            axle_top_right.SetTextureFilename("carbon");
            axle_top_right.setShadeMode(eShadeMode.smFlat);
            axle_top_right.setFlatColor(Color.White);

            childNode.setPrimitive(axle_top_right);
            xf = new UWB_XformInfo();
            xf = childNode.getXformInfo();
            pivPos = Vector3.Divide((min + max), 2);
            pivPos.X = min.X;
            childNode.originalPivot = pivPos;
            xf.SetPivot(pivPos);
            childNode.setXformInfo(xf);
            m_sRoot.insertChildNode(childNode);

            {
                // Construct the top tire for this axle
                subChildNode = new UWB_SceneNode("axle_top_right_tire_top");
                UWB_PrimitiveCircle axle_top_right_tire_top = new UWB_PrimitiveCircle();
                center = new Vector3(90.7f, 26.0f, 0.0f);
                radius = 0.5f;
                axle_top_right_tire_top.setCenter(center);
                axle_top_right_tire_top.setRadius(radius);

                axle_top_right_tire_top.EnableBlending(true);
                axle_top_right_tire_top.EnableTexturing(true);
                axle_top_right_tire_top.SetTextureFilename("yellowtire");
                axle_top_right_tire_top.setShadeMode(eShadeMode.smFlat);
                axle_top_right_tire_top.setFlatColor(Color.White);

                subChildNode.setPrimitive(axle_top_right_tire_top);
                xf = new UWB_XformInfo();
                xf = subChildNode.getXformInfo();
                subChildNode.originalPivot = pivPos;
                xf.SetPivot(new Vector3(90.7f, 25.5f, 0.0f));
                subChildNode.setXformInfo(xf);
                childNode.insertChildNode(subChildNode);

                // Construct the bottom tire for this axle
                subChildNode = new UWB_SceneNode("axle_top_right_tire_btm");
                UWB_PrimitiveCircle axle_top_right_tire_btm = new UWB_PrimitiveCircle();
                center = new Vector3(90.7f, 25.0f, 0.0f);
                radius = 0.5f;
                axle_top_right_tire_btm.setCenter(center);
                axle_top_right_tire_btm.setRadius(radius);

                axle_top_right_tire_btm.EnableBlending(true);
                axle_top_right_tire_btm.EnableTexturing(true);
                axle_top_right_tire_btm.SetTextureFilename("yellowtire");
                axle_top_right_tire_btm.setShadeMode(eShadeMode.smFlat);
                axle_top_right_tire_btm.setFlatColor(Color.White);

                subChildNode.setPrimitive(axle_top_right_tire_btm);
                xf = new UWB_XformInfo();
                xf = subChildNode.getXformInfo();
                subChildNode.originalPivot = pivPos;
                xf.SetPivot(new Vector3(90.7f, 25.5f, 0.0f));
                subChildNode.setXformInfo(xf);
                childNode.insertChildNode(subChildNode);
            }

            // Construct the arm of our hero
            childNode = new UWB_SceneNode("arm_bottom");
            UWB_PrimitiveList armandBall = new UWB_PrimitiveList();
            UWB_PrimitiveRectangle arm_bottom = new UWB_PrimitiveRectangle();
            min = new Vector3(85.2f, 29.0f, 0.0f);
            max = new Vector3(88.2f, 30.0f, 0.0f);
            arm_bottom.SetCorners(min, max);

            arm_bottom.EnableBlending(true);
            arm_bottom.EnableTexturing(true);
            arm_bottom.SetTextureFilename("arm");
            arm_bottom.setShadeMode(eShadeMode.smFlat);
            arm_bottom.setFlatColor(Color.White);

            // create movingCircle
            movingCircle = new UWB_PrimitiveCircle();
            center = new Vector3((min.X + max.X) / 2, (min.Y + max.Y) / 2, 0);
            radius = .75f;
            movingCircle.setCenter(center);
            movingCircle.setRadius(radius);
            movingCircle.setFlatColor(Color.Black);
            movingCircle.setShadingColor(Color.Red);
            movingCircle.setVelocity(new Vector3(4.0f, 0, 0));
            movingCircle.EnableBlending(true);
            movingCircle.EnableTexturing(true);
            movingCircle.SetTextureFilename("chinared");
            movingCircle.setShadeMode(eShadeMode.smFlat);
            movingCircle.setFlatColor(Color.White);
            armandBall.Append(arm_bottom);
            armandBall.Append(movingCircle);
            childNode.setPrimitive(armandBall);
            xf = new UWB_XformInfo();
            xf = childNode.getXformInfo();
            pivPos = Vector3.Divide((min + max), 2);
            pivPos.Y = min.Y;
            childNode.originalPivot = pivPos;
            xf.SetPivot(pivPos);
            childNode.setXformInfo(xf);
            m_sRoot.insertChildNode(childNode);

            {
                // Construct the arm extension on the arm
                subChildNode = new UWB_SceneNode("arm_extension");
                UWB_PrimitiveRectangle arm_extension = new UWB_PrimitiveRectangle();
                min = new Vector3(86.2f, 30.0f, 0.0f);
                max = new Vector3(87.2f, 32.0f, 0.0f);
                arm_extension.SetCorners(min, max);

                arm_extension.EnableBlending(true);
                arm_extension.EnableTexturing(true);
                arm_extension.SetTextureFilename("armExt");
                arm_extension.setShadeMode(eShadeMode.smFlat);
                arm_extension.setFlatColor(Color.White);

                subChildNode.setPrimitive(arm_extension);
                xf = new UWB_XformInfo();
                xf = subChildNode.getXformInfo();
                pivPos = Vector3.Divide((min + max), 2);
                pivPos.Y = min.Y;
                subChildNode.originalPivot = pivPos;
                xf.SetPivot(pivPos);
                subChildNode.setXformInfo(xf);
                childNode.insertChildNode(subChildNode);

                {

                    // Construct the magnet
                    childNode = subChildNode;

                    subChildNode = new UWB_SceneNode("magnet");

                    UWB_PrimitiveList magnet = new UWB_PrimitiveList();
                    UWB_PrimitiveCircle arm_joint = new UWB_PrimitiveCircle();

                    arm_joint.EnableBlending(true);
                    arm_joint.EnableTexturing(true);
                    arm_joint.SetTextureFilename("chinared");
                    arm_joint.setShadeMode(eShadeMode.smFlat);
                    arm_joint.setFlatColor(Color.White);

                    center = new Vector3(86.7f, 32.5f, 0.0f);
                    radius = 0.5f;
                    arm_joint.setCenter(center);
                    arm_joint.setRadius(radius);

                    // Construct the bottom portion of the magnet
                    UWB_PrimitiveRectangle magnet_btm = new UWB_PrimitiveRectangle();
                    min = new Vector3(85.2f, 33.0f, 0.0f);
                    max = new Vector3(88.2f, 37.0f, 0.0f);
                    magnet_btm.EnableBlending(true);
                    magnet_btm.EnableTexturing(true);
                    magnet_btm.SetTextureFilename("carbon");
                    magnet_btm.setShadeMode(eShadeMode.smFlat);
                    magnet_btm.setFlatColor(Color.White);
                    magnet_btm.SetCorners(min, max);
                    /*
                    // Construct the left extension of the magnet
                    UWB_PrimitiveRectangle magnet_left = new UWB_PrimitiveRectangle();
                    min = new Vector3(85.2f, 34.0f, 0.0f);
                    max = new Vector3(86.2f, 35.0f, 0.0f);
                    magnet_left.EnableBlending(true);
                    magnet_left.EnableTexturing(true);
                    magnet_left.SetTextureFilename("carbon");
                    magnet_left.setShadeMode(eShadeMode.smFlat);
                    magnet_left.setFlatColor(Color.White);
                    magnet_left.SetCorners(min, max);

                    // Construct the right extension of the magnet
                    UWB_PrimitiveRectangle magnet_right = new UWB_PrimitiveRectangle();
                    min = new Vector3(87.2f, 34.0f, 0.0f);
                    max = new Vector3(88.2f, 35.0f, 0.0f);
                    magnet_right.EnableBlending(true);
                    magnet_right.EnableTexturing(true);
                    magnet_right.SetTextureFilename("carbon");
                    magnet_right.setShadeMode(eShadeMode.smFlat);
                    magnet_right.setFlatColor(Color.White);
                    magnet_right.SetCorners(min, max);
                    */
                    // Hide the primitive circles in our magnet
                    m_magnetcollisionBtm = new UWB_PrimitiveCircle();
                    center = new Vector3(86.7f, 33.5f, 0);
                    radius = 1.25f;
                    m_magnetcollisionBtm.setCenter(center);
                    m_magnetcollisionBtm.setRadius(radius);
                    m_magnetcollisionBtm.setVisible(false);

                    m_magnetcollisionLeft = new UWB_PrimitiveCircle();
                    center = new Vector3(85.7f, 34.5f, 0);
                    radius = 0.75f;
                    m_magnetcollisionLeft.setCenter(center);
                    m_magnetcollisionLeft.setRadius(radius);
                    m_magnetcollisionLeft.setVisible(false);

                    m_magnetcollisionRight = new UWB_PrimitiveCircle();
                    center = new Vector3(87.7f, 34.5f, 0);
                    radius = 0.75f;
                    m_magnetcollisionRight.setCenter(center);
                    m_magnetcollisionRight.setRadius(radius);
                    m_magnetcollisionRight.setVisible(false);

                    // Hide a point for direction calculation
                    m_front = new UWB_PrimitivePoint();
                    center = new Vector3(86.7f, 34f, 0);
                    m_front.setPointSize(1);
                    m_front.setLocation(center);
                    m_front.setVisible(false);

                    magnet.Append(arm_joint);
                    magnet.Append(magnet_btm);
                    //  magnet.Append(magnet_left);
                    //  magnet.Append(magnet_right);
                    magnet.Append(m_magnetcollisionBtm);
                    magnet.Append(m_magnetcollisionLeft);
                    magnet.Append(m_magnetcollisionRight);

                    subChildNode.setPrimitive(magnet);

                    xf = new UWB_XformInfo();
                    xf = subChildNode.getXformInfo();
                    pivPos = new Vector3(86.7f, 32.5f, 0.0f);
                    //pivPos.Y -= radius;
                    subChildNode.originalPivot = pivPos;
                    xf.SetPivot(pivPos);

                    subChildNode.setXformInfo(xf);
                    childNode.insertChildNode(subChildNode);

                }

                childNode = subChildNode;
                subChildNode = new UWB_SceneNode("wave");
                UWB_PrimitiveList wave = new UWB_PrimitiveList();

                center = new Vector3(86.7f, 36f, 0);
                radius = 3.00f;
                m_magwavebtm = new UWB_PrimitiveCircle();
                m_magwavebtm.setCenter(center);
                m_magwavebtm.setRadius(radius);
                m_magwavebtm.setFillMode(eFillMode.fmWireframe);
                m_magwavebtm.setVisible(false);

                center = new Vector3(86.7f, 39.75f, 0);
                m_magwavemdl = new UWB_PrimitiveCircle();
                m_magwavemdl.setCenter(center);
                m_magwavemdl.setRadius(radius + 0.25f);
                m_magwavemdl.setFillMode(eFillMode.fmWireframe);
                m_magwavemdl.setVisible(false);

                center = new Vector3(86.7f, 42f, 0);
                m_magwavetop = new UWB_PrimitiveCircle();
                m_magwavetop.setCenter(center);
                m_magwavetop.setRadius(radius + 0.50f);
                m_magwavetop.setFillMode(eFillMode.fmWireframe);
                m_magwavetop.setVisible(false);

                wave.Append(m_magwavebtm);
                wave.Append(m_magwavemdl);
                wave.Append(m_magwavetop);

                subChildNode.setPrimitive(wave);

                xf = new UWB_XformInfo();
                xf = subChildNode.getXformInfo();
                pivPos = new Vector3(86.7f, 32.5f, 0.0f);
                //pivPos.Y -= radius;
                subChildNode.originalPivot = pivPos;
                xf.SetPivot(pivPos);

                subChildNode.setXformInfo(xf);
                childNode.insertChildNode(subChildNode);

            }
        }

        public void draw(eLevelofDetail lod, UWB_DrawHelper drawHelper)
        {
            drawHelper.InitializeModelTransform();

            // Draw the magnet wave before hero
            // Push the original top of the model xform stack so we don't lose it
            drawHelper.PushModelTransform();
            // Traverse down to magnet level and accumulate all model transforms
            m_sRoot.searchNodeByName("Body").getXformInfo().SetUpModelStack(drawHelper);
            m_sRoot.searchNodeByName("arm_bottom").getXformInfo().SetUpModelStack(drawHelper);
            m_sRoot.searchNodeByName("arm_extension").getXformInfo().SetUpModelStack(drawHelper);
            m_sRoot.searchNodeByName("magnet").getXformInfo().SetUpModelStack(drawHelper);
            drawHelper.resetAttributes();
            drawHelper.setFillMode(eFillMode.fmWireframe);
            //m_MagnetWaveBox.Draw(drawHelper);
            for (int i = 0; i < m_points.Count; i++)
                m_points[i].draw(lod, drawHelper);

            drawHelper.PopModelTransform();

            drawHelper.PushModelTransform();

            m_sRoot.draw(lod, drawHelper);

            drawHelper.PopModelTransform();
        }

        public void updateMovingCircle(float elapsedSeconds)
        {
            // Move circle back and forth within arm
            //  if (movingCircle.getLocation().X <= 85.2f || movingCircle.getLocation().X >= 88.2f)
            //      movingCircle.setVelocity(movingCircle.getVelocity() * -1);
            if (movingCircle.getLocation().X < 85.7f)
            {
                movingCircle.setCenter(new Vector3(85.71f, movingCircle.getLocation().Y, 0));
                Vector3 newVel = new Vector3(movingCircle.getVelocity().X * -1f, 0, 0);
                movingCircle.setVelocity(newVel);
            }
            if (movingCircle.getLocation().X > 87.7f)
            {
                movingCircle.setCenter(new Vector3(87.69f, movingCircle.getLocation().Y, 0));
                Vector3 newVel = new Vector3(movingCircle.getVelocity().X * -1f, 0, 0);
                movingCircle.setVelocity(newVel);
            }
            movingCircle.update(elapsedSeconds);
        }

        public void PlayAnimation(float elapsedSeconds)
        {
            UWB_XformInfo xf = new UWB_XformInfo();
            xf = m_sRoot.searchNodeByName("axle_btm_left_tire_top").getXformInfo();
            xf.SetRotationInRadians(xf.GetRotationInRadians() + (float)Math.Sin(1) * elapsedSeconds * 20);
            m_sRoot.searchNodeByName("axle_btm_left_tire_top").setXformInfo(xf);

            xf = m_sRoot.searchNodeByName("axle_btm_left_tire_btm").getXformInfo();
            xf.SetRotationInRadians(xf.GetRotationInRadians() + (float)Math.Sin(1) * elapsedSeconds * 20);
            m_sRoot.searchNodeByName("axle_btm_left_tire_btm").setXformInfo(xf);

            xf = m_sRoot.searchNodeByName("axle_mid_left_tire_top").getXformInfo();
            xf.SetRotationInRadians(xf.GetRotationInRadians() + (float)Math.Sin(1) * elapsedSeconds * 20);
            m_sRoot.searchNodeByName("axle_mid_left_tire_top").setXformInfo(xf);

            xf = m_sRoot.searchNodeByName("axle_mid_left_tire_btm").getXformInfo();
            xf.SetRotationInRadians(xf.GetRotationInRadians() + (float)Math.Sin(1) * elapsedSeconds * 20);
            m_sRoot.searchNodeByName("axle_mid_left_tire_btm").setXformInfo(xf);

            xf = m_sRoot.searchNodeByName("axle_top_left_tire_top").getXformInfo();
            xf.SetRotationInRadians(xf.GetRotationInRadians() + (float)Math.Sin(1) * elapsedSeconds * 20);
            m_sRoot.searchNodeByName("axle_top_left_tire_top").setXformInfo(xf);

            xf = m_sRoot.searchNodeByName("axle_top_left_tire_btm").getXformInfo();
            xf.SetRotationInRadians(xf.GetRotationInRadians() + (float)Math.Sin(1) * elapsedSeconds * 20);
            m_sRoot.searchNodeByName("axle_top_left_tire_btm").setXformInfo(xf);

            xf = m_sRoot.searchNodeByName("axle_btm_right_tire_top").getXformInfo();
            xf.SetRotationInRadians(xf.GetRotationInRadians() + (float)Math.Sin(1) * elapsedSeconds * -20);
            m_sRoot.searchNodeByName("axle_btm_right_tire_top").setXformInfo(xf);

            xf = m_sRoot.searchNodeByName("axle_btm_right_tire_btm").getXformInfo();
            xf.SetRotationInRadians(xf.GetRotationInRadians() + (float)Math.Sin(1) * elapsedSeconds * -20);
            m_sRoot.searchNodeByName("axle_btm_right_tire_btm").setXformInfo(xf);

            xf = m_sRoot.searchNodeByName("axle_mid_right_tire_top").getXformInfo();
            xf.SetRotationInRadians(xf.GetRotationInRadians() + (float)Math.Sin(1) * elapsedSeconds * -20);
            m_sRoot.searchNodeByName("axle_mid_right_tire_top").setXformInfo(xf);

            xf = m_sRoot.searchNodeByName("axle_mid_right_tire_btm").getXformInfo();
            xf.SetRotationInRadians(xf.GetRotationInRadians() + (float)Math.Sin(1) * elapsedSeconds * -20);
            m_sRoot.searchNodeByName("axle_mid_right_tire_btm").setXformInfo(xf);

            xf = m_sRoot.searchNodeByName("axle_top_right_tire_top").getXformInfo();
            xf.SetRotationInRadians(xf.GetRotationInRadians() + (float)Math.Sin(1) * elapsedSeconds * -20);
            m_sRoot.searchNodeByName("axle_top_right_tire_top").setXformInfo(xf);

            xf = m_sRoot.searchNodeByName("axle_top_right_tire_btm").getXformInfo();
            xf.SetRotationInRadians(xf.GetRotationInRadians() + (float)Math.Sin(1) * elapsedSeconds * -20);
            m_sRoot.searchNodeByName("axle_top_right_tire_btm").setXformInfo(xf);

        }

        public void CreateSuctionEffect()
        {
            m_rand = new Random();
            int num = m_rand.Next(1, 10);
            UWB_PrimitivePoint point;
            for (int i = 0; i < num; i++)
            {
                m_rand = new Random();
                float x = (float)m_rand.Next((int)m_MagnetWaveBox.GetMin().X, (int)m_MagnetWaveBox.GetMax().X);
                float y = (float)m_rand.Next((int)m_MagnetWaveBox.GetMin().Y, (int)m_MagnetWaveBox.GetMax().Y);
                float size = (float)m_rand.NextDouble() * 5.0f;
                point = new UWB_PrimitivePoint();
                point.setLocation(new Vector3(x, y, 0));
                point.setPointSize(size);
                int pick = m_rand.Next(1, 3);

                switch (pick)
                {
                    case 1:
                        point.SetTextureFilename("armExt");
                        break;
                    case 2:
                        point.SetTextureFilename("yellowtire");
                        break;
                    case 3:
                        point.SetTextureFilename("bodyblue");
                        break;
                    default:
                        point.SetTextureFilename("armExt");
                        break;
                }


                point.EnableBlending(true);
                point.EnableTexturing(true);
                m_points.Add(point);
            }
        }

        public void SuctionAnimation(float elapsedSeconds)
        {
            if (m_points.Count < 20)
                CreateSuctionEffect();

            for (int i = 0; i < m_points.Count; i++)
            {
                Vector3 direction = Vector3.Normalize(m_front.getLocation() - m_points[i].getLocation());
                m_points[i].setLocation(m_points[i].getLocation() + direction * elapsedSeconds);
            }

            for (int i = 0; i < m_points.Count; i++)
            {
                if (!m_MagnetWaveBox.ContainsPoint(m_points[i].getLocation()))
                {
                    m_points.RemoveAt(i);
                    //break;
                }
            }
        }

        public void BlowAnimation(float elapsedSeconds)
        {
            if (m_points.Count < 20)
                CreateSuctionEffect();

            for (int i = 0; i < m_points.Count; i++)
            {
                Vector3 direction = Vector3.Normalize(m_points[i].getLocation() - m_front.getLocation());
                m_points[i].setLocation(m_points[i].getLocation() + direction * elapsedSeconds);
            }

            for (int i = 0; i < m_points.Count; i++)
            {
                if (!m_MagnetWaveBox.ContainsPoint(m_points[i].getLocation()))
                {
                    m_points.RemoveAt(i);
                    break;
                }
            }
        }

        public void ClearPoints()
        {
            m_points.Clear();
        }

        public void DeathAnimation(float elapsedSeconds)
        {
            UWB_XformInfo xf = m_sRoot.getXformInfo();
            xf.SetRotationInDegrees(xf.GetRotatioinInDegrees() + (float)Math.Sin(1) * elapsedSeconds * 10);
            xf.SetScale(xf.GetScale() * 1.01f);
            m_sRoot.setXformInfo(xf);
        }

        public Vector3 isCollidingWithMovingCircle(Vector3 center, Vector3 point, UWB_DrawHelper drawHelper
            , Vector3 currVelocity, ref bool collide)
        {
            Vector3 circP1 = movingCircle.getLocation();
            Vector3 circP2 = new Vector3(movingCircle.getLocation().X + movingCircle.getRadius(),
                movingCircle.getLocation().Y + movingCircle.getRadius(), 0);

            // Push the original top of the model xform stack so we don't lose it
            drawHelper.PushModelTransform();
            m_sRoot.searchNodeByName("Body").getXformInfo().SetUpModelStack(drawHelper);
            m_sRoot.searchNodeByName("arm_bottom").getXformInfo().SetUpModelStack(drawHelper);
            drawHelper.TransformPoint(ref circP1);
            drawHelper.TransformPoint(ref circP2);
            // get rid of this xform information to restore stack to its original state
            drawHelper.PopModelTransform();

            // now we have all the points we need to test for collision
            // Perform simple circle collision test
            if ((Vector3.Distance(center, circP1)) <= ((Vector3.Distance(circP2, circP1))
                + Vector3.Distance(center, point)))
            {
                Vector3 newV = new Vector3();
                currVelocity.Length();
                newV = Vector3.Multiply(Vector3.Normalize(Vector3.Subtract(center, circP1)),
                    currVelocity.Length());
                collide = true;
                return newV;
            }
            collide = false;
            return currVelocity;
        }

        // test the 2 points passed into this function with magnet's collision circles
        // to see if the 2 objects collide
        public bool isCollidingWithMagnet(Vector3 center, Vector3 point, UWB_DrawHelper drawHelper)
        {
            Vector3 m_magnetBtmP1 = m_magnetcollisionBtm.getLocation();
            Vector3 m_magnetBtmP2 = new Vector3(m_magnetcollisionBtm.getLocation().X + m_magnetcollisionBtm.getRadius(),
                m_magnetcollisionBtm.getLocation().Y + m_magnetcollisionBtm.getRadius(), 0);
            Vector3 m_magnetLftP1 = m_magnetcollisionLeft.getLocation();
            Vector3 m_magnetLftP2 = new Vector3(m_magnetcollisionLeft.getLocation().X + m_magnetcollisionLeft.getRadius(),
                m_magnetcollisionLeft.getLocation().Y + m_magnetcollisionLeft.getRadius(), 0);
            Vector3 m_magnetRgtP1 = m_magnetcollisionRight.getLocation();
            Vector3 m_magnetRgtP2 = new Vector3(m_magnetcollisionRight.getLocation().X + m_magnetcollisionRight.getRadius(),
                m_magnetcollisionRight.getLocation().Y + m_magnetcollisionRight.getRadius(), 0);

            // For calculation the direction of our hero
            Vector3 front = m_front.getLocation();

            // Push the original top of the model xform stack so we don't lose it
            drawHelper.PushModelTransform();

            // Traverse down to magnet level and accumulate all model transforms
            m_sRoot.searchNodeByName("Body").getXformInfo().SetUpModelStack(drawHelper);
            m_sRoot.searchNodeByName("arm_bottom").getXformInfo().SetUpModelStack(drawHelper);
            m_sRoot.searchNodeByName("arm_extension").getXformInfo().SetUpModelStack(drawHelper);
            m_sRoot.searchNodeByName("magnet").getXformInfo().SetUpModelStack(drawHelper);

            // transform all the points of interest to world coordinate with the
            // transform info we have accumulated
            drawHelper.TransformPoint(ref m_magnetBtmP1);
            drawHelper.TransformPoint(ref m_magnetBtmP2);
            drawHelper.TransformPoint(ref m_magnetLftP1);
            drawHelper.TransformPoint(ref m_magnetLftP2);
            drawHelper.TransformPoint(ref m_magnetRgtP1);
            drawHelper.TransformPoint(ref m_magnetRgtP2);
            drawHelper.TransformPoint(ref front);

            MagnetBound.Center = front;
            MagnetBound.Radius = 1;

            // get rid of this xform information to restore stack to its original state
            drawHelper.PopModelTransform();

            HeroDirection = Vector3.Normalize(front - m_magnetBtmP1);

            // now we have all the points we need to test for collision
            // Perform simple circle collision test
            if ((Vector3.Distance(center, m_magnetBtmP1)) <= ((Vector3.Distance(m_magnetBtmP2, m_magnetBtmP1))
                + Vector3.Distance(center, point)))
                return true;
            else if ((Vector3.Distance(center, m_magnetLftP1)) <= ((Vector3.Distance(m_magnetLftP2, m_magnetLftP1))
                + Vector3.Distance(center, point)))
                return true;
            else if ((Vector3.Distance(center, m_magnetRgtP1)) <= ((Vector3.Distance(m_magnetRgtP2, m_magnetRgtP1))
                + Vector3.Distance(center, point)))
                return true;
            else
                return false;
        }


        public Vector3 GetHeroDirection()
        {
            return HeroDirection;
        }


        public bool isInMagnetWaveBounds(Vector3 center, Vector3 point, UWB_DrawHelper drawHelper)
        {
            Vector3 m_magnetBtmP1 = m_magwavebtm.getLocation();
            Vector3 m_magnetBtmP2 = new Vector3(m_magnetBtmP1.X + m_magwavebtm.getRadius(),
                m_magnetBtmP1.Y + m_magwavebtm.getRadius(), 0);
            Vector3 m_magnetMdlP1 = m_magwavemdl.getLocation();
            Vector3 m_magnetMdlP2 = new Vector3(m_magnetMdlP1.X + m_magwavemdl.getRadius(),
                m_magnetMdlP1.Y + m_magwavemdl.getRadius(), 0);
            Vector3 m_magnetTopP1 = m_magwavetop.getLocation();
            Vector3 m_magnetTopP2 = new Vector3(m_magnetTopP1.X + m_magwavetop.getRadius(),
                m_magnetTopP1.Y + m_magwavetop.getRadius(), 0);

            // Push the original top of the model xform stack so we don't lose it
            drawHelper.PushModelTransform();

            // Traverse down to magnet level and accumulate all model transforms
            m_sRoot.searchNodeByName("Body").getXformInfo().SetUpModelStack(drawHelper);
            m_sRoot.searchNodeByName("arm_bottom").getXformInfo().SetUpModelStack(drawHelper);
            m_sRoot.searchNodeByName("arm_extension").getXformInfo().SetUpModelStack(drawHelper);
            m_sRoot.searchNodeByName("magnet").getXformInfo().SetUpModelStack(drawHelper);

            // transform all the points of interest to world coordinate with the
            // transform info we have accumulated
            drawHelper.TransformPoint(ref m_magnetBtmP1);
            drawHelper.TransformPoint(ref m_magnetBtmP2);
            drawHelper.TransformPoint(ref m_magnetMdlP1);
            drawHelper.TransformPoint(ref m_magnetMdlP2);
            drawHelper.TransformPoint(ref m_magnetTopP1);
            drawHelper.TransformPoint(ref m_magnetTopP2);

            drawHelper.PopModelTransform();

            //Vector3 center1 = new Vector3(m_magnetBtmP1.X, m_magnetBtmP1.Y, 0);
            //m_magwavebtm.Center = center1;

            // now we have all the points we need to test for collision
            // Perform simple circle collision test
            if ((Vector3.Distance(center, m_magnetBtmP1)) <= ((Vector3.Distance(m_magnetBtmP2, m_magnetBtmP1))
                + Vector3.Distance(center, point)))
                return true;
            else if ((Vector3.Distance(center, m_magnetMdlP1)) <= ((Vector3.Distance(m_magnetMdlP2, m_magnetMdlP1))
                + Vector3.Distance(center, point)))
                return true;
            else if ((Vector3.Distance(center, m_magnetTopP1)) <= ((Vector3.Distance(m_magnetTopP2, m_magnetTopP1))
                + Vector3.Distance(center, point)))
                return true;
            else
                return false;
        }

        private float speed;
        private float rotation;
        private float magnetFuel;
        private float consumptionRate; // current fuel consumption rate

        // changes the consumption rate of the magnet, fuel gets decremented in update()
        public void changeConsumption(float rate)
        {
            if (rate > .01)
                consumptionRate = rate * -2.0f;    // consumption is negative
            else
                consumptionRate = 0;

        }

        public bool hasFuel()
        {
            return (magnetFuel > 0.001f);
        }

        // Functions to support hero movement
        public void SetSpeed(float s)
        {
            speed = s * .2f;
        }

        public void setRotation(float rot)
        {
            rotation = getRootNode().getXformInfo().GetRotatioinInDegrees();
            rotation += rot;
            UWB_XformInfo xf = getRootNode().getXformInfo();
            xf.SetRotationInDegrees(rotation);
            getRootNode().setXformInfo(xf);
        }

        public void setMagnetRotation(float rot)
        {
            rotation = getRootNode().searchNodeByName("arm_bottom").getXformInfo().GetRotatioinInDegrees();
            rotation += rot;
            rotation = MathHelper.Clamp(rotation, -90, 90);
            UWB_XformInfo xf = getRootNode().searchNodeByName("arm_bottom").getXformInfo();
            xf.SetRotationInDegrees(rotation);
            getRootNode().searchNodeByName("arm_bottom").setXformInfo(xf);
        }

        // the change in velocity due to user control
        private Vector3 velocityDeltaController(float elapsedSeconds)
        {
            float x = (float)(speed *
                    Math.Cos(getRootNode().getXformInfo().GetRotationInRadians()));
            float y = (float)(speed *
                    Math.Sin(getRootNode().getXformInfo().GetRotationInRadians()));
            float xp = (float)(x * Math.Cos(UWB_Math.UWB_ToRadians(90)) +
                y * Math.Sin(UWB_Math.UWB_ToRadians(90)));
            float yp = (float)(y * Math.Cos(UWB_Math.UWB_ToRadians(90)) - x *
                Math.Sin(UWB_Math.UWB_ToRadians(90)));

            return new Vector3(xp, yp, 0);
        }

        private Vector3 vChangeByModel;

        // used for instantaneous changes in velocity by model
        // it should be set to 0 if model doesnt need to do anything
        public void setVelocityDeltaModel(Vector3 vm)
        {
            vChangeByModel = vm;
        }


        public void Update(float elapsedSeconds)
        {
            Vector3 vChangeByUser = velocityDeltaController(elapsedSeconds);
            Vector3 currentVel = getRootNode().getVelocity();
            // maximum speed allowed (noobish way)
            /*if (currentVel.X > 30)
                currentVel.X = 30;
            if (currentVel.Y > 30)
                currentVel.Y = 30;
            if (currentVel.X < -30)
                currentVel.X = -30;
            if (currentVel.Y < -30)
                currentVel.Y = -30;*/

            // elegant way
            currentVel.X = MathHelper.Clamp(currentVel.X, -50, 50);
            currentVel.Y = MathHelper.Clamp(currentVel.Y, -50, 50);

            // slow down hero by .95 each second
            currentVel.Y *= (1f - .5f * elapsedSeconds);    // friction slows down hero
            currentVel.X *= (1f - .5f * elapsedSeconds);
            getRootNode().setVelocity(currentVel + vChangeByModel + vChangeByUser);
            getRootNode().moveNodeByVelocity(elapsedSeconds);
            vChangeByModel = new Vector3(); // reset to 0 since change is already applied

            // change fuel of magnet, if it actually has some
            if (magnetFuel > 0)
                magnetFuel += consumptionRate * elapsedSeconds;
            if (magnetFuel < 14)
                magnetFuel += .7f * elapsedSeconds;
            int sign;
            if (movingCircle.getVelocity().X >= 0)
                sign = 1;
            else
                sign = -1;
            movingCircle.setVelocity(new Vector3(sign * 4.0f * (magnetFuel / 7f), 0, 0));
            updateMovingCircle(elapsedSeconds);
        }

        public bool isStationary()
        {
            return true;
            //  return (m_sRoot.getVelocity().X <= UWB_Utility.zeroTolerance
            //          && m_sRoot.getVelocity().Y <= UWB_Utility.zeroTolerance
            //          && m_sRoot.getVelocity().Z <= UWB_Utility.zeroTolerance);
        }

        // cheap hero bound implementation
        public UWB_BoundingCircle GetHeroBound(UWB_DrawHelper drawHelper)
        {
            Vector3 center = m_sRoot.getXformInfo().GetPivot();


            // Push the original top of the model xform stack so we don't lose it
            drawHelper.PushModelTransform();

            m_sRoot.searchNodeByName("Body").getXformInfo().SetUpModelStack(drawHelper);

            drawHelper.TransformPoint(ref center);

            drawHelper.PopModelTransform();

            HeroBound.Center = center;
            HeroBound.Radius = 6;
            BigHeroBound.Center = center;
            BigHeroBound.Radius = 25;

            return HeroBound;
        }

        public UWB_BoundingCircle GetMagnetBound(UWB_DrawHelper drawHelper)
        {
            return MagnetBound;
        }

        // Used to increase effiency by checking to see if the point of interest is within hero bound or not
        public bool isInHeroCoverageArea(Vector3 pointofinterest)
        {
            return BigHeroBound.ContainsPoint(pointofinterest);
        }

        // DEBUGGING
        public void DrawHeroBound(UWB_DrawHelper drawHelper)
        {
            drawHelper.resetAttributes();
            drawHelper.setFillMode(eFillMode.fmWireframe);
            drawHelper.setShadeMode(eShadeMode.smFlat);
            drawHelper.setColor1(Color.Red);

            //drawHelper.DrawCircle(HeroBound.Center, HeroBound.Radius);
            //drawHelper.DrawCircle(MagnetBound.Center, MagnetBound.Radius);
            HeroBound.Draw(drawHelper);
        }

        public void HeroWorldBoundResponse(int i)
        {
            // UWB_XformInfo xf = getRootNode().getXformInfo();
            // Vector3 trans = xf.GetTranslation();
            Vector3 currentVel = getRootNode().getVelocity();
            switch (i)
            {
                case 0:     // left wall
                    //trans.X = -80.7f;
                    currentVel.X = currentVel.X * -1;
                    break;
                case 1:     // right wall
                    //trans.X = 357.3f;
                    currentVel.X = currentVel.X * -1;
                    break;
                case 2:     // bottom
                    //trans.Y = -17;
                    currentVel.Y = currentVel.Y * -1;
                    break;
                case 3:     // top
                    //trans.Y = 103;
                    currentVel.Y = currentVel.Y * -1;
                    break;
                default:
                    break;
            }
            getRootNode().setVelocity(currentVel);
            // xf.SetTranslation(trans);

            // getRootNode().setXformInfo(xf);
        }

        public Vector3 GetFront(UWB_DrawHelper drawHelper)
        {
            Vector3 front = m_front.getLocation();

            // Push the original top of the model xform stack so we don't lose it
            drawHelper.PushModelTransform();

            // Traverse down to magnet level and accumulate all model transforms
            m_sRoot.searchNodeByName("Body").getXformInfo().SetUpModelStack(drawHelper);
            m_sRoot.searchNodeByName("arm_bottom").getXformInfo().SetUpModelStack(drawHelper);
            m_sRoot.searchNodeByName("arm_extension").getXformInfo().SetUpModelStack(drawHelper);
            m_sRoot.searchNodeByName("magnet").getXformInfo().SetUpModelStack(drawHelper);

            drawHelper.TransformPoint(ref front);

            drawHelper.PopModelTransform();

            return front;
        }

        public bool AnimateWave
        {
            get { return animateWave; }
            set { animateWave = value; }
        }

    }
}