// University of Washington Bothell Graphics Library
// Author: Timothy Chuang
// Based on Kelvin Sung and Steve Baer's D3D Library 
// and Robert Stone's XNA Library version 5
// The accompanying library supports CSS Graphics courses taught at UW-Bothell
// See: http://courses.washington.edu/css450/
//      http://courses.washington.edu/css451/
///////////////////////////////////////////////////////////////////////////////////

using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework;


namespace MagGame
{
    public enum eLevelofDetail { lodLow, lodMed, lodHigh };
    public enum eShadeMode { smFlat, smGouraud };
    public enum eFillMode { fmPoint, fmWireframe, fmSolid };

    public class UWB_DrawHelper
    {
        protected eLevelofDetail mLod;
        protected eShadeMode mShadeMode;
        protected eFillMode mFillMode;

        protected float mPointSize;
        protected Color mColor1;
        protected Color mColor2;
        protected string m_TexFileName;

        public UWB_DrawHelper()
        {
            mLod = eLevelofDetail.lodHigh;
            mShadeMode = eShadeMode.smGouraud;
            mFillMode = eFillMode.fmSolid;
            m_TexFileName = null;
        }

        public void resetAttributes()
        {
            mPointSize = 1.0f;
            mColor1 = Color.Black;
            mColor2 = Color.Black;
        }

        public void setLod(eLevelofDetail lod)
        {
            mLod = lod;
        }

        public eLevelofDetail getLod()
        {
            return mLod;
        }

        public virtual void setShadeMode(eShadeMode shadeMode)
        {
            mShadeMode = shadeMode;
        }

        public eShadeMode getShadeMode()
        {
            return mShadeMode;
        }

        public virtual void setFillMode(eFillMode fillMode)
        {
            mFillMode = fillMode;
        }

        public eFillMode getFillMode()
        {
            return mFillMode;
        }

        public Color setColor1(Color color1)
        {
            Color oldColor = mColor1;
            mColor1 = color1;
            return oldColor;
        }

        public Color setColor2(Color color2)
        {
            Color oldColor = mColor2;
            mColor2 = color2;
            return oldColor;
        }

        public float setPointSize(float ptSize)
        {
            float oldPtSize = mPointSize;
            mPointSize = ptSize;
            return oldPtSize;
        }


        // Subclasses return true if they support the draw operation
        // The base class just always returns false for all draw functions
        public virtual bool DrawPoint(Vector3 position)
        {
            return false;
        }

        public virtual bool DrawLine(Vector3 start, Vector3 end)
        {
            return false;
        }

        public virtual bool DrawCircle(Vector3 center, float radius)
        {
            return false;
        }

        public virtual bool DrawRectangleXY(Vector3 corner1, Vector3 corner2)
        {
            return false;
        }
        public virtual bool AccumulateModelTransform(
            Vector3 translation,
            Vector3 scale,
            float rotationRadians,
            Vector3 pivot)
        {

            return false;
        }

        public virtual bool PushModelTransform()
        {
            return false;
        }

        public virtual bool PopModelTransform()
        {
            return false;
        }

        public virtual bool InitializeModelTransform()
        {
            return false;
        }

        public virtual bool TransformPoint(ref Vector3 point)
        {
            return false;
        }
        public virtual void SetTextureInfo(string t) { m_TexFileName = t; }
        public virtual bool EnableTexture(bool on) { return false; }
        public virtual bool EnableBlending(bool on) { return false; }
    }
}
