// University of Washington Bothell Graphics Library
// Modified by: Timothy Chuang
// Based on Kelvin Sung and Steve Baer's D3D Library 
// and Robert Stone's XNA Library version 5
// The accompanying library supports CSS Graphics courses taught at UW-Bothell
// See: http://courses.washington.edu/css450/
//      http://courses.washington.edu/css451/
///////////////////////////////////////////////////////////////////////////////////

using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace MagGame
{
	public class UWB_Primitive
	{
		protected bool mVisible;
		protected Vector3 mVelocity;
		protected Color mFlatColor;
		protected Color mShadingColor;
		protected eFillMode mFillMode;
		protected eShadeMode mShadeMode;
		protected float mPointSize;
        protected bool m_bDrawBoundingVolume;
        public bool removeMe; // used to determine if ball needs delete in model

        protected string m_TexFilename;
        protected bool m_bTexturingEnabled;
        protected bool m_bBlendingEnabled;

		public UWB_Primitive()
		{
			mVisible = true;
			mVelocity = Vector3.Zero;
			mFlatColor = Color.Black;
			mShadingColor = Color.White;
			mFillMode = eFillMode.fmSolid;
			mShadeMode = eShadeMode.smGouraud;
			mPointSize = 2f;
            m_bDrawBoundingVolume = false;
            removeMe = false;
		}

		public void draw(eLevelofDetail lod, UWB_DrawHelper drawHelper)
		{
			if (!mVisible)
				return;

			setupDrawAttributes(drawHelper);
			drawPrimitive(lod, drawHelper);

            if (IsDrawBoundingVolume())
                DrawBoundingVolume(lod, drawHelper, Color.Red);
		}

		public virtual void update(float elapsedSeconds)
		{

		}

		public virtual void mouseDownVertex(int vertexID, float x, float y)
		{

		}

		public virtual void moveTo(float x, float y)
		{

		}

		public virtual Vector3 getLocation()
		{
			return Vector3.Zero;
		}

		public void setVisible(bool visible)
		{
			mVisible = visible;
		}

		public bool isVisible()
		{
			return mVisible;
		}

		public void setVelocity(Vector3 velocity)
		{
			mVelocity = velocity;
		}

		public Vector3 getVelocity()
		{
			return mVelocity;
		}

		public bool isStationary()
		{
			return (mVelocity.X <= UWB_Utility.zeroTolerance
					&& mVelocity.Y <= UWB_Utility.zeroTolerance
					&& mVelocity.Z <= UWB_Utility.zeroTolerance);
		}

		public void setFlatColor(Color color)
		{
			mFlatColor = color;
		}

		public Color getFlatColor()
		{
			return mFlatColor;
		}

		public void setShadingColor(Color color)
		{
			mShadingColor = color;
		}

		public Color getShadingColor()
		{
			return mShadingColor;
		}

		public void setFillMode(eFillMode fillMode)
		{
			mFillMode = fillMode;
		}

		public eFillMode getFillMode()
		{
			return mFillMode;
		}

		public void setShadeMode(eShadeMode shadeMode)
		{
			mShadeMode = shadeMode;
		}

		public eShadeMode getShadeMode()
		{
			return mShadeMode;
		}

		public void setPointSize(float size)
		{
			mPointSize = size;
		}

		public float getPointSize()
		{
			return mPointSize;
		}

		protected virtual void setupDrawAttributes(UWB_DrawHelper drawHelper)
		{
            drawHelper.setColor1(mFlatColor);
            drawHelper.setColor2(mShadingColor);
            drawHelper.SetTextureInfo(m_TexFilename);
            drawHelper.EnableBlending(m_bBlendingEnabled);
            drawHelper.EnableTexture(m_bTexturingEnabled);
		}

		protected virtual void drawPrimitive(eLevelofDetail lod, UWB_DrawHelper drawHelper)
		{

		}

        public virtual UWB_BoundingVolume GetBoundingVolume(eLevelofDetail lod)
        {
            return null;
        }

        public void CollisionResponse(ref UWB_Primitive pOther, Vector3 other_location)
        {
            Vector3 dir = getLocation() - other_location;   // this vector is along the location of the two primitives
            
            dir.Normalize();

            float mySpeed = getVelocity().Length();

            setVelocity(mySpeed * dir);
        }

        public void DrawBoundingVolume(eLevelofDetail lod, UWB_DrawHelper pRenderer, Color color)
        {
            UWB_BoundingVolume pBounds = GetBoundingVolume(lod);
            if (pBounds != null && pRenderer != null)
            {
                pRenderer.resetAttributes();
                pRenderer.setColor1(color);
                pRenderer.setShadeMode(eShadeMode.smFlat);
                pRenderer.setFillMode(eFillMode.fmWireframe);
                pBounds.Draw(pRenderer);
            }
        }

        public void SetDrawBoundingVolume(bool on)
        {
            m_bDrawBoundingVolume = on;
        }

        public bool IsDrawBoundingVolume()
        {
            return m_bDrawBoundingVolume;
        }


        public void SetTextureFilename(string t) { m_TexFilename = t; }

        public string GetTextureFilename() { return m_TexFilename; }

        public void EnableTexturing(bool on) { m_bTexturingEnabled = on; }
        public bool IsTexturingEnabled() { return m_bTexturingEnabled; }
        public void EnableBlending(bool on) { m_bBlendingEnabled = on; }
        public bool IsBlendingEnabled() { return m_bBlendingEnabled; }
	}
}
