// University of Washington Bothell Graphics Library
// Modified by: Timothy Chuang
// Based on Kelvin Sung and Steve Baer's D3D Library 
// and Robert Stone's XNA Library version 5
// The accompanying library supports CSS Graphics courses taught at UW-Bothell
// See: http://courses.washington.edu/css450/
//      http://courses.washington.edu/css451/
///////////////////////////////////////////////////////////////////////////////////

using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace MagGame
{
    public class UWB_PrimitiveCircle: UWB_Primitive
    {
		private Vector3 mCenter;
		private float mRadius;
        private UWB_BoundingBox m_bounds;


		public UWB_PrimitiveCircle()
		{
			mCenter = Vector3.Zero;
			mRadius = 0;
            m_bounds = new UWB_BoundingBox();


            // always keep the bounds in synch with the circle geometry
            Vector3 corner1 = new Vector3(mCenter.X - mRadius, mCenter.Y - mRadius, 0);
            Vector3 corner2 = new Vector3(mCenter.X + mRadius, mCenter.Y + mRadius, 0);
            m_bounds.SetCorners(corner1, corner2);

		}

		protected override void setupDrawAttributes(UWB_DrawHelper drawHelper)
		{
			base.setupDrawAttributes(drawHelper);
			drawHelper.setColor1(mFlatColor);
			drawHelper.setColor2(mShadingColor);
			drawHelper.setShadeMode(mShadeMode);
			drawHelper.setFillMode(mFillMode);
		}

		protected override void drawPrimitive(eLevelofDetail lod, UWB_DrawHelper drawHelper)
		{
			eLevelofDetail oldLod = drawHelper.getLod();
			drawHelper.setLod(lod);
			drawHelper.DrawCircle(mCenter, mRadius);
			drawHelper.setLod(oldLod);
		}

		public override void mouseDownVertex(int vertexID, float x, float y)
		{
			if (vertexID == 0)
			{
				setCenter(x, y, 0f);
				setRadius(0);
			}
			else
			{
				float dx = x - mCenter.X;
				float dy = y - mCenter.Y;
				float radius = (float)Math.Sqrt(dx * dx + dy * dy);
				setRadius(radius);
			}
		}

		public void setCenter(Vector3 center)
		{
			mCenter = center;

            // always keep the bounds in synch with the circle geometry
            Vector3 corner1 = new Vector3(mCenter.X - mRadius, mCenter.Y - mRadius, 0);
            Vector3 corner2 = new Vector3(mCenter.X + mRadius, mCenter.Y + mRadius, 0);
            m_bounds.SetCorners(corner1, corner2);
		}

		public void setCenter(float x, float y, float z)
		{
			mCenter.X = x;
			mCenter.Y = y;
			mCenter.Z = z;
		}

        public override void moveTo(float x, float y)
        {
            setCenter(x, y, 0);
        }

		public override Vector3 getLocation()
		{
			return mCenter;
		}

		public void setRadius(float radius)
		{
			mRadius = radius;

            // always keep the bounds in synch with the circle geometry
            Vector3 corner1 = new Vector3(mCenter.X - mRadius, mCenter.Y - mRadius, 0);
            Vector3 corner2 = new Vector3(mCenter.X + mRadius, mCenter.Y + mRadius, 0);
            m_bounds.SetCorners(corner1, corner2);
		}

		public float getRadius()
		{
			return mRadius;
		}

		public override void update(float elapsedSeconds)
		{
			Vector3 adjVelocity = mVelocity * elapsedSeconds;
			mCenter += adjVelocity;

            // always keep the bounds in synch with the circle geometry
            Vector3 corner1 = new Vector3(mCenter.X - mRadius, mCenter.Y - mRadius, 0);
            Vector3 corner2 = new Vector3(mCenter.X + mRadius, mCenter.Y + mRadius, 0);
            m_bounds.SetCorners(corner1, corner2);

		}

        public override UWB_BoundingVolume GetBoundingVolume(eLevelofDetail lod)
        {
            return m_bounds;
        }
    }
}
