// University of Washington Bothell Graphics Library
// Modified by: Timothy Chuang
// Based on Kelvin Sung and Steve Baer's D3D Library 
// and Robert Stone's XNA Library version 5
// The accompanying library supports CSS Graphics courses taught at UW-Bothell
// See: http://courses.washington.edu/css450/
//      http://courses.washington.edu/css451/
///////////////////////////////////////////////////////////////////////////////////

using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace MagGame
{
    public class UWB_PrimitiveLine: UWB_Primitive
    {
		protected Vector3 mStart;
		protected Vector3 mEnd;
        private UWB_BoundingBox m_bounds;

		public UWB_PrimitiveLine()
		{
			mStart = Vector3.Zero;
			mEnd = Vector3.Zero;
            m_bounds = new UWB_BoundingBox();
		}

		protected override void setupDrawAttributes(UWB_DrawHelper drawHelper)
		{
			base.setupDrawAttributes(drawHelper);
			drawHelper.setColor1(mFlatColor);
			drawHelper.setColor2(mShadingColor);
			drawHelper.setShadeMode(mShadeMode);
		}

		protected override void drawPrimitive(eLevelofDetail lod, UWB_DrawHelper drawHelper)
		{
			drawHelper.DrawLine(mStart, mEnd);

		}

		public override void mouseDownVertex(int vertexID, float x, float y)
		{
			if (vertexID == 0)
				setStartPoint(x, y, 0f);

			setEndPoint(x, y, 0f);
		}

		public override void moveTo(float x, float y)
		{
			setEndPoint(x, y, 0f);
            //Update bounding box
            m_bounds.SetCorners(mStart, mEnd);
		}

		private Vector3 getMidPoint()
		{
			Vector3 m = (mStart + mEnd) / 2f;
			return m;
		}

		public override Vector3 getLocation()
		{
			return getMidPoint();
		}

		public void setStartPoint(float x, float y, float z)
		{
			mStart.X = x;
			mStart.Y = y;
			mStart.Z = z;
		}

		public Vector3 getStartPoint()
		{
			return mStart;
		}

		public void setEndPoint(float x, float y, float z)
		{
			mEnd.X = x;
			mEnd.Y = y;
			mEnd.Z = z;
            //Update bounding box
            m_bounds.SetCorners(mStart, mEnd);
		}

		public Vector3 getEndPoint()
		{
			return mEnd;
		}

		public override void update(float elapsedSeconds)
		{
			Vector3 adjVelocity = mVelocity * elapsedSeconds;
			mStart += adjVelocity;
			mEnd += adjVelocity;
            //Update bounding box
            m_bounds.SetCorners(mStart, mEnd);
		}

        public override UWB_BoundingVolume GetBoundingVolume(eLevelofDetail lod)
        {
            return m_bounds;
        }
    }
}
