// University of Washington Bothell Graphics Library
// Author: George Sohal
// Based on Kelvin Sung and Steve Baer's D3D Library 
// and Robert Stone's XNA Library version 5
// The accompanying library supports CSS Graphics courses taught at UW-Bothell
// See: http://courses.washington.edu/css450/
//      http://courses.washington.edu/css451/
///////////////////////////////////////////////////////////////////////////////////

using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace MagGame
{
    public class UWB_PrimitiveList : UWB_Primitive
    {
        private List<UWB_Primitive> m_list;
        private UWB_BoundingBox m_bounds;
        
        // DEBUG - delete this later
        //private UWB_XformInfo m_xform;

        public UWB_PrimitiveList()
        {
            m_list = new List<UWB_Primitive>();
            m_bounds = new UWB_BoundingBox();

            //m_xform = new UWB_XformInfo();
        }

        protected override void drawPrimitive(eLevelofDetail lod, UWB_DrawHelper drawHelper)
        {
            //drawHelper.PopModelTransform();
            //drawHelper.InitializeModelTransform();
            //drawHelper.PushModelTransform();

            //m_xform.SetUpModelStack(drawHelper);

            int count = m_list.Count;
            for (int i = 0; i < count; i++)
            {
                m_list[i].draw(lod, drawHelper);
            }

            //drawHelper.PopModelTransform();

        }

        // DEBUG - delete this later
        /*public void setXform(UWB_XformInfo x)
        {
            m_xform.SetPivot(x.GetPivot());
            m_xform.SetRotationInDegrees(x.GetRotatioinInDegrees());
            m_xform.SetRotationInRadians(x.GetRotationInRadians());
            m_xform.SetScale(x.GetScale());
            m_xform.SetTranslation(x.GetTranslation());
        }*/

        public void Update(float elapsed_seconds)
        {
            int count = m_list.Count;
            for (int i = 0; i < count; i++)
            {
                m_list[i].update(elapsed_seconds);
            }
        }

        public override UWB_BoundingVolume GetBoundingVolume(eLevelofDetail lod)
        {
            int count = m_list.Count;
            if (count == 0)
                return null;
            else
            {
                UWB_PrimitiveList pList = this;
                pList.m_bounds.MakeInvalid();

                for (int i = 0; i < count; i++)
                {
                    UWB_Primitive pChild = pList.m_list[i];
                    UWB_BoundingBox pBV = (UWB_BoundingBox) pChild.GetBoundingVolume(lod);
                    pList.m_bounds.Add(pBV);
                } 
                m_bounds = pList.m_bounds;
            }
            return m_bounds;
        }


        public int Count()
        {
            return m_list.Count;
        }

        public void PrimitiveAt(int index, ref UWB_Primitive yourPrimitive)
        {
            if (index < 0 || index >= m_list.Count)
                return;
            yourPrimitive = m_list[index];
        }

        public void Append(UWB_Primitive pPrimitive)
        {
            m_list.Add(pPrimitive);
            m_bounds.Add(pPrimitive.GetBoundingVolume(eLevelofDetail.lodHigh));
        }

        public void DeletePrimitiveAt(int index)
        {
            m_list.RemoveAt(index);
        }

        public void DrawChildBoundingVolumes(eLevelofDetail lod,
            UWB_DrawHelper pDrawHelper, Color color) 
        {
            if (pDrawHelper == null)
                return;
            int count = Count();
            for (int i=0; i<count; i++)
            {
                m_list[i].DrawBoundingVolume(lod, pDrawHelper, color);
            }
        }
    }
}
