// University of Washington Bothell Graphics Library
// Modified by: Timothy Chuang
// Based on Kelvin Sung and Steve Baer's D3D Library 
// and Robert Stone's XNA Library version 5
// The accompanying library supports CSS Graphics courses taught at UW-Bothell
// See: http://courses.washington.edu/css450/
//      http://courses.washington.edu/css451/
///////////////////////////////////////////////////////////////////////////////////

using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace MagGame
{
    public class UWB_PrimitivePoint: UWB_Primitive
    {
        protected Vector3 mPoint;
        private UWB_BoundingBox m_bounds;

		public UWB_PrimitivePoint()
        {
            mPoint = Vector3.Zero;
            m_bounds = new UWB_BoundingBox();
        }

		public UWB_PrimitivePoint(float x, float y, float z)
		{
			mPoint = new Vector3(x, y, z);
            m_bounds.SetCorners(mPoint, mPoint);
		}

		public override void update(float elapsedSeconds)
		{
			Vector3 adjVelocity = mVelocity * elapsedSeconds;
			mPoint += adjVelocity;
            m_bounds.SetCorners(mPoint, mPoint);
		}

		public override void mouseDownVertex(int vertexID, float x, float y)
		{
			setLocation(x, y, 0);
		}

		public override void moveTo(float x, float y)
		{
			setLocation(x, y, 0);
		}

		protected override void setupDrawAttributes(UWB_DrawHelper drawHelper)
		{
			base.setupDrawAttributes(drawHelper);
			drawHelper.setColor1(mFlatColor);
			drawHelper.setPointSize(mPointSize);
		}

		protected override void drawPrimitive(eLevelofDetail lod, UWB_DrawHelper drawHelper)
		{
			drawHelper.DrawPoint(mPoint);
		}

        public void setLocation(float x, float y, float z)
        {
			mPoint.X = x;
			mPoint.Y = y;
			mPoint.Z = z;

            m_bounds.SetCorners(mPoint, mPoint);
        }

		public void setLocation(Vector3 location)
		{
			mPoint.X = location.X;
			mPoint.Y = location.Y;
			mPoint.Z = location.Z;

            m_bounds.SetCorners(mPoint, mPoint);
		}

        public override Vector3 getLocation()
        {
            return mPoint;
        }

        public override UWB_BoundingVolume GetBoundingVolume(eLevelofDetail lod)
        {
            return m_bounds;
        }
    }
}
