// University of Washington Bothell Graphics Library
// Author: George Sohal
// Based on Kelvin Sung and Steve Baer's D3D Library 
// and Robert Stone's XNA Library version 5
// The accompanying library supports CSS Graphics courses taught at UW-Bothell
// See: http://courses.washington.edu/css450/
//      http://courses.washington.edu/css451/
///////////////////////////////////////////////////////////////////////////////////

using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace MagGame
{
    public class UWB_PrimitiveRectangle : UWB_Primitive
    {
        float m_width;
        float m_height;
        UWB_BoundingBox m_bounds;
        Vector3 mCenter;
        Vector3 m_mouse_down_point;

        public UWB_PrimitiveRectangle()
        {
            m_width = m_height = 0;
            mCenter = new Vector3();
            m_bounds = new UWB_BoundingBox();
        }

        protected override void setupDrawAttributes(UWB_DrawHelper drawHelper)
        {
            base.setupDrawAttributes(drawHelper);
            drawHelper.setColor1(mFlatColor);
            drawHelper.setColor2(mShadingColor);
            drawHelper.setShadeMode(mShadeMode);
            drawHelper.setFillMode(mFillMode);
        }

        protected override void drawPrimitive(eLevelofDetail lod, UWB_DrawHelper drawHelper)
        {
            drawHelper.DrawRectangleXY(m_bounds.GetMin(), m_bounds.GetMax());
        }

        public override void mouseDownVertex(int vertexID, float x, float y)
        {
            if (vertexID == 0)
            {
                m_mouse_down_point = new Vector3(x, y, 0);
                m_bounds.SetCorners(m_mouse_down_point, m_mouse_down_point);
            }
            else
            {
                m_bounds.SetCorners(m_mouse_down_point, new Vector3(x, y, 0));
            }
        }

        public void MoveTo(float x, float y)
        {
            Vector3 old_center = m_bounds.GetCenter();
            float dx = x - old_center.X;
            float dy = y - old_center.Y;

            Vector3 min_point = m_bounds.GetMin();
            Vector3 max_point = m_bounds.GetMax();

            min_point.X += dx;
            min_point.Y += dy;
            max_point.X += dx;
            max_point.Y += dy;
            m_bounds.SetCorners(min_point, max_point);
        }

        public Vector3 GetLocation()
        {
            return m_bounds.GetCenter();
        }

        public void SetCorners(Vector3 corner1, Vector3 corner2)
        {
            m_bounds.SetCorners(corner1, corner2);
        }

        public void Update(float elapsed_seconds)
        {
            Vector3 adjust_vec = mVelocity * elapsed_seconds;
            Vector3 min_point = m_bounds.GetMin();
            Vector3 max_point = m_bounds.GetMax();

            min_point += adjust_vec;
            max_point += adjust_vec;
            m_bounds.SetCorners(min_point, max_point);
        }

        public override UWB_BoundingVolume GetBoundingVolume(eLevelofDetail lod)
        {
            return m_bounds;
        }


    }
}
