// University of Washington Bothell Graphics Library
// Authors: Timothy Chuang
// Based on Kelvin Sung and Steve Baer's D3D Library 
// and Robert Stone's XNA Library version 5
// The accompanying library supports CSS Graphics courses taught at UW-Bothell
// See: http://courses.washington.edu/css450/
//      http://courses.washington.edu/css451/
///////////////////////////////////////////////////////////////////////////////////

using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace MagGame
{
    public class UWB_XformInfo
    {
        protected Vector3 m_pivot;
        protected Vector3 m_translation;
        protected Vector3 m_scale;
        protected float m_rotation_radians;

        public UWB_XformInfo()
        {
            m_pivot = new Vector3(0f, 0f, 0f);
            this.ToIdentiy();
        }

        public void SetUpModelStack(UWB_DrawHelper draw_helper)
        {
            draw_helper.AccumulateModelTransform(m_translation, m_scale, m_rotation_radians, m_pivot);
        }

        public void ToIdentiy()
        {
            m_translation = new Vector3(0f, 0f, 0f);
            m_scale = new Vector3(1f, 1f, 1f);
            m_rotation_radians = 0.0f;
        }

        public void DrawPivot(UWB_DrawHelper h, float size)
        {
            h.resetAttributes();
            h.setColor1(Color.White);
            h.setColor2(Color.Green);
            h.setFillMode(eFillMode.fmSolid);
            h.setShadeMode(eShadeMode.smGouraud);
            h.DrawCircle(m_pivot, size);
        }

        public Vector3 GetTranslation()
        {
            return m_translation;
        }

        public void SetTranslation(Vector3 translation)
        {
            m_translation = translation;
        }

        public Vector3 GetScale()
        {
            return m_scale;
        }

        public void SetScale(Vector3 scale)
        {
            m_scale = scale;
        }

        public float GetRotationInRadians()
        {
            return m_rotation_radians;
        }

        public float GetRotatioinInDegrees()
        {
            return UWB_Math.UWB_ToDegrees(m_rotation_radians);
        }

        public void SetRotationInRadians(float radians)
        {
            m_rotation_radians = radians;
        }

        public void SetRotationInDegrees(float degrees)
        {
            m_rotation_radians = UWB_Math.UWB_ToRadians(degrees);
        }

        public Vector3 GetPivot()
        {
            return m_pivot;
        }

        public void SetPivot(Vector3 pivot)
        {
            m_pivot = pivot;
        }        
    }
}
